def abs(self, ir="", ia="", name="", facta="", **kwargs):
    """Forms the absolute value of a variable.

    APDL Command: ABS

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting
        variable (2 to NV [NUMVAR]).  If this number is the same
        as for a previously defined variable, the previously
        defined variable will be overwritten with this result.

    ia
        Reference number of the variable to be operated on.

    name
        Thirty-two character name for identifying the variable on
        the printout and displays.  Embedded blanks are compressed
        upon output.

    facta
        Scaling factor (positive or negative) applied to variable
        IA (defaults to 1.0).

    Notes
    -----
    The new variable is calculated as:

    IR = | FACTA x IA |

    For a complex number (a + ib), the absolute value is the
    magnitude, where the IA values are obtained from:
    ``sqrt(a**2 + b**2)``

    See POST26 - Data Operations in the Mechanical APDL Theory
    Reference for details.
    """
    return self.run(f"ABS,{ir},{ia},,,{name},,,{facta}", **kwargs)


def add(self, ir="", ia="", ib="", ic="", name="", facta="", factb="",
        factc="", **kwargs):
    """Adds (sums) variables.

    APDL Command: ADD

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting variable (2 to
        NV [NUMVAR]).  If this number is the same as for a previously
        defined variable, the previously defined variable will be
        overwritten with this result.

    ia, ib, ic
        Reference numbers of the three variables to be operated on.  If
        only two variables, leave IC blank.  If only one, leave IB and IC
        blank.

    name
        Thirty-two character name for identifying the variable on the
        printout and displays.  Embedded blanks are compressed upon output.

    facta, factb, factc
        Scaling factors (positive or negative) applied to the corresponding
        variables (default to 1.0).

    Notes
    -----
    Adds variables (up to three at once) according to the operation:

    IR = (FACTA x IA) + (FACTB x IB) + (FACTC x IC)
    """
    command = f"ADD,{ir},{ia},{ib},{ic},{name},,,{facta},{factb},{factc}"
    return self.run(command, **kwargs)


def atan(self, ir="", ia="", name="", facta="", **kwargs):
    """Forms the arctangent of a complex variable.

    APDL Command: ATAN

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting variable (2 to
        NV [NUMVAR]).  If this number is the same as for a previously
        defined variable, the previously defined variable will be
        overwritten with this result.

    ia
        Reference number of the complex variable to be operated on.

    name
        Thirty-two character name for identifying the variable on the
        printout and displays.  Embedded blanks are compressed upon output.

    facta
        Scaling factor (positive or negative) applied to variable
        IA (defaults to 1.0).  Usually FACTA should be set to 1.
        FACTA may affect the position of the angle by a multiple
        of π, resulting in a quadrant change.

    Notes
    -----
    Forms the arctangent of a complex variable according to the
    operation:

    IR = ATAN(FACTA X b/a)

    where a and b are the real and imaginary parts, respectively,
    of the complex variable IA (which is of the form a + ib).  The
    arctangent represents the phase angle (in radians), and is
    valid only for a harmonic analysis (ANTYPE,HARMIC).

    Since the scaling factor is applied uniformly to b/a, applying
    any positive or negative scaling factor will not affect the
    size of the phase angle, with the exception that a negative
    scaling factor will change the results quadrant by : π.  The
    magnitude of a complex number is still obtained through the
    ABS command.  See POST26 - Data Operations in the Mechanical
    APDL Theory Reference for details.
    """
    command = f"ATAN,{ir},{ia},,,{name},,,{facta}"
    return self.run(command, **kwargs)


def clog(self, ir="", ia="", name="", facta="", factb="", **kwargs):
    """Forms the common log of a variable

    APDL Command: CLOG

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting
        variable (2 to NV [NUMVAR]).  If this number is the same
        as for a previously defined variable, the previously
        defined variable will be overwritten with this result.

    ia
        Reference number of the variable to be operated on.

    name
        Thirty-two character name for identifying the variable on
        printouts and displays.  Embedded blanks are compressed
        for output.

    facta
        Scaling factor applied to variable IA (defaults to 1.0).

    factb
        Scaling factor (positive or negative) applied to the operation
        (defaults to 1.0).

    Notes
    -----
    Forms the common log of a variable according to the operation:

    IR = FACTB*LOG(FACTA x IA)
    """
    return self.run(f"CLOG,{ir},{ia},,,{name},,,{facta},{factb}", **kwargs)


def conjug(self, ir="", ia="", name="", facta="", **kwargs):
    """Forms the complex conjugate of a variable.

    APDL Command: CONJUG

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting variable (2 to
        NV [NUMVAR]).  If this number is the same as for a previously
        defined variable, the previously defined variable will be
        overwritten with this result.

    ia
        Reference number of the variable to be operated on.

    name
        Thirty-two character name for identifying the variable on printouts
        and displays.  Embedded blanks are compressed for output.

    facta
        Scaling factor (positive or negative) applied to variable (default
        to 1.0).

    Notes
    -----
    Used only with harmonic analyses (ANTYPE,HARMIC).
    """
    return self.run(f"CONJUG,{ir},{ia},,,{name},,,{facta}", **kwargs)


def deriv(self, ir="", iy="", ix="", name="", facta="", **kwargs):
    """Differentiates a variable.

    APDL Command: DERIV

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting
        variable (2 to NV [NUMVAR]).  If this number is the same
        as for a previously defined variable, the previously
        defined variable will be overwritten with this result.

    iy, ix
        Reference numbers of variables to be operated on.  IY is
        differentiated with respect to IX.

    name
        Thirty-two character name for identifying the variable on
        printouts and displays. Embedded blanks are compressed for
        output.

    facta
        Scaling factor (positive or negative) applied as shown
        below (defaults to 1.0).

    Notes
    -----
    Differentiates variables according to the operation:

    IR = FACTA x d(IY)/d(IX)
    """
    return self.run(f"DERIV,{ir},{iy},{ix},,{name},,,{facta}", **kwargs)


def exp(self, ir="", ia="", name="", facta="", factb="", **kwargs):
    """Forms the exponential of a variable.

    APDL Command: EXP

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting
        variable (2 to NV [NUMVAR]). If this number is the same as
        for a previously defined variable, the previously defined
        variable will be overwritten with this result.

    ia
        Reference number of the variable to be operated on.

    name
        Thirty-two character name for identifying the variable on
        the printout and displays. Embedded blanks are compressed
        upon output.

    facta
        Scaling factor applied to variable IA (defaults to 1.0).

    factb
        Scaling factor (positive or negative) applied to the operation
        (defaults to 1.0).

    Notes
    -----
    Forms the exponential of a variable according to the operation:

    ``IR = FACTB*EXP(FACTA x IA)``
    """
    return self.run(f"EXP,{ir},{ia},,,{name},,,{facta},{factb}", **kwargs)


def filldata(self, ir="", lstrt="", lstop="", linc="", value="", dval="",
             **kwargs):
    """Fills a variable by a ramp function.

    APDL Command: FILLDATA

    Parameters
    ----------
    ir
        Define data table as variable IR (2 to NV [NUMVAR]).

    lstrt
        Start at location LSTRT (defaults to 1).

    lstop
        Stop at location LSTOP (defaults to maximum location as determined
        from data previously stored.

    linc
        Fill every LINC location between LSTRT and LSTOP (defaults to 1).

    value
        Value assigned to location LSTRT.

    dval
        Increment value of previous filled location by DVAL and assign sum
        to next location to be filled (may be positive or negative.)

    Notes
    -----
    Locations may be filled continuously or at regular intervals (LINC).
    Previously defined data at a location will be overwritten.
    """
    command = f"FILLDATA,{ir},{lstrt},{lstop},{linc},{value},{dval}"
    return self.run(command, **kwargs)


def imagin(self, ir="", ia="", name="", facta="", **kwargs):
    """Forms an imaginary variable from a complex variable.

    APDL Command: IMAGIN

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting
        variable (2 to NV [NUMVAR]).  If this number is the same
        as for a previously defined variable, the previously
        defined variable will be overwritten with this result.

    ia
        Reference number of the variable to be operated on.

    name
        Thirty-two character name for identifying the variable on
        the printout and displays.  Embedded blanks are compressed
        upon output.

    facta
        Scaling factor (positive or negative) applied to variable IA
        (defaults to 1.0).

    Notes
    -----
    This command forms a new variable from a complex variable by
    storing the imaginary part as the real part.  The imaginary
    part can then be used in other operations.  Used only with
    harmonic analyses (ANTYPE,HARMIC).

    Complex variables are stored in two-column arrays with the
    real component stored in the first column and the imaginary
    component stored in the second column.  This command extracts
    the value stored in the second column (i.e., imaginary
    component).  However, with harmonic analyses, all variables
    are stored in two-column arrays as complex variables.  If the
    variable is not complex, then the same value is stored in both
    columns.  This command will extract the variable in the second
    column of the array, even if this variable is not the
    imaginary component of a complex variable.
    """
    return self.run(f"IMAGIN,{ir},{ia},,,{name},,,{facta}", **kwargs)


def int1(self, ir="", iy="", ix="", name="", facta="", factb="", const="",
         **kwargs):
    """Integrates a variable.

    APDL Command: INT1

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting
        variable (2 to NV [NUMVAR]).  If this number is the same
        as for a previously defined variable, the previously
        defined variable will be overwritten with this result.
        Table values represent integrated sum of IY to current
        table position of IX.

    iy, ix
        Integrate variable IY with respect to IX.

    name
        Thirty-two character name for identifying the variable on
        the printout and displays.  Embedded blanks are compressed
        upon output.

    facta, factb
        Scaling factors (positive or negative) applied to the
        corresponding variables (default to 1.0).

    const
        Initial value.

    Notes
    -----
    Integrates variables according to the operation:

    IR = ∫ (FACTA x IY) d(FACTB x IX) + CONST
    """
    command = f"INT1,{ir},{iy},{ix},,{name},,,{facta},{factb},{const}"
    return self.run(command, **kwargs)


def large(self, ir="", ia="", ib="", ic="", name="", facta="", factb="",
          factc="", **kwargs):
    """Finds the largest (the envelope) of three variables.

    APDL Command: LARGE

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting variable (2 to
        NV [NUMVAR]).  If this number is the same as for a previously
        defined variable, the previously defined variable will be
        overwritten with this result.

    ia, ib, ic
        Reference numbers of the three variables to be operated on.  If
        only two, leave IC blank.  If only one, leave IB blank also.

    name
        Thirty-two character name for identifying the variable on the
        printout and displays.  Embedded blanks are compressed upon output.

    facta, factb, factc
        Scaling factors (positive or negative) applied to the corresponding
        variables (default to 1.0).

    Notes
    -----
    Creates a new variable by finding the largest of up to three variables
    according to the operation:

    IR = Largest of (FACTA x IA, FACTB x IB, FACTC x IC)

    The comparison is done at each time location, so that the new variable
    is the "envelope" of the three existing variables.
    """
    command = f"LARGE,{ir},{ia},{ib},{ic},{name},,,{facta},{factb},{factc}"
    return self.run(command, **kwargs)


def nlog(self, ir="", ia="", name="", facta="", factb="", **kwargs):
    """Forms the natural log of a variable.

    APDL Command: NLOG

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting
        variable (2 to NV [NUMVAR]).  If this number is the same
        as for a previously defined variable, the previously
        defined variable will be overwritten with this result.

    ia
        Reference number of the variable to be operated on.

    name
        Thirty-two character name identifying the variable on
        printouts and displays.  Embedded blanks are compressed
        for output.

    facta
        Scaling factor applied to variable IA (defaults to 1.0).

    factb
        Scaling factor (positive or negative) applied to the operation
        (defaults to 1.0).

    Notes
    -----
    Forms the natural log of a variable according to the operation:

    IR = FACTB*LN(FACTA x IA)
    """
    return self.run(f"NLOG,{ir},{ia},,,{name},,,{facta},{factb}", **kwargs)


def prod(self, ir="", ia="", ib="", ic="", name="", facta="", factb="",
         factc="", **kwargs):
    """Multiplies variables.

    APDL Command: PROD

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting
        variable (2 to NV [NUMVAR]).  If this number is the same
        as for a previously defined variable, the previously
        defined variable will be overwritten with this result.

    ia, ib, ic
        Reference numbers of the three variables to be operated
        on.  If only two leave IC blank.  If only one, leave IB
        blank also.

    name
        Thirty-two character name identifying the variable on
        printouts and displays.  Embedded blanks are compressed
        for output.

    facta, factb, factc
        Scaling factors (positive or negative) applied to the
        corresponding variables (default to 1.0).

    Notes
    -----
    Multiplies variables (up to three at once) according to the
    operation:

    ``IR = (FACTA x IA) x (FACTB x IB) x (FACTC x IC)``
    """
    return self.run(f"PROD,{ir},{ia},{ib},{ic},{name},,,{facta},{factb},{factc}",
                    **kwargs)


def quot(self, ir="", ia="", ib="", name="", facta="", factb="", **kwargs):
    """Divides two variables.

    APDL Command: QUOT

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting variable (2 to
        NV [NUMVAR]).  If this number is the same as for a previously
        defined variable, the previously defined variable will be
        overwritten with this result.

    ia, ib
        Reference numbers of the two variables to be operated on.

    name
        Thirty-two character name identifying the variable on printouts and
        displays.  Embedded blanks are compressed for output.

    facta, factb
        Scaling factors (positive or negative) applied to the corresponding
        variables (default to 1.0).

    Notes
    -----
    Divides two variables according to the operation:

    IR = (FACTA x IA)/(FACTB x IB)
    """
    return self.run(f"QUOT,{ir},{ia},{ib},,{name},,,{facta},{factb}", **kwargs)


def realvar(self, ir="", ia="", name="", facta="", **kwargs):
    """Forms a variable using only the real part of a complex variable.

    APDL Command: REALVAR

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting variable (2 to
        NV [NUMVAR]).  If this number is the same as for a previously
        defined variable, the previously defined variable will be
        overwritten with this result.

    ia
        Reference number of the variable to be operated on.

    name
        Thirty-two character name identifying the variable on printouts and
        displays.  Embedded blanks are compressed for output.

    facta
        Scaling factor (positive or negative) applied to variable IA
        (defaults to 1.0).

    Notes
    -----
    Forms a variable using only the real part of a variable.  Used only
    with harmonic analyses (ANTYPE,HARMIC).

    Complex variables are stored in two-column arrays with the
    real component stored in the first column and the imaginary
    component stored in the second column.  This command extracts
    the value stored in the first column (i.e., real component).
    However with harmonic analyses, all variables are stored in
    two-column arrays as complex variables.  If the variable is
    not complex, then the same value is stored in both columns.
    This command will extract the variable in the first column of
    the array, even if this variable is not the real component of
    a complex variable.
    """
    command = f"REALVAR,{ir},{ia},,,{name},,,{facta}"
    return self.run(command, **kwargs)


def small(self, ir="", ia="", ib="", ic="", name="", facta="", factb="",
          factc="", **kwargs):
    """Finds the smallest of three variables.

    APDL Command: SMALL

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting
        variable (2 to NV [NUMVAR]).  If this number is the same
        as for a previously defined variable, the previously
        defined variable will be overwritten with this result.

    ia, ib, ic
        Reference numbers of the three variables to be operated
        on.  If only two, leave IC blank.  If only one, leave IB
        blank also.

    name
        Thirty-two character name identifying the variable on
        printouts and displays.  Embedded blanks are compressed
        for output.

    facta, factb, factc
        Scaling factors (positive or negative) applied to the
        corresponding variables (defaults to 1.0).

    Notes
    -----
    Finds the smallest of three variables according to the operation:

    ``IR = min(FACTA x IA, FACTB x IB, FACTC x IC)``
    """
    command = f"SMALL,{ir},{ia},{ib},{ic},{name},,,{facta},{factb},{factc}"
    return self.run(command, **kwargs)


def sqrt(self, ir="", ia="", name="", facta="", **kwargs):
    """Forms the square root of a variable.

    APDL Command: SQRT

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting variable (2 to
        NV [NUMVAR]).  If this number is the same as for a previously
        defined variable, the previously defined variable will be
        overwritten with this result.

    ia
        Reference number of the variable to be operated on.

    name
        Thirty-two character name identifying the variable on printouts and
        displays.  Embedded blanks are compressed for output.

    facta
        Scaling factor (positive or negative) applied to variable IA
        (defaults to 1.0).

    Notes
    -----
    Forms the square root of a variable according to the operation:
    ``IR=sqrt(FACTA*IA)``
    """
    return self.run(f"SQRT,{ir},{ia},,,{name},,,{facta}", **kwargs)



from . import twod_to_3d_analysis
from . import all_others
from . import analysis_options
from . import birth_and_death
from . import dynamic_options
from . import fe_body_loads
from . import fe_constraints
from . import fe_forces
from . import fe_surface_loads
from . import gap_conditions
from . import inertia
from . import load_step_operations
from . import load_step_options
from . import master_dof
from . import miscellaneous_loads
from . import multi_field_solver_convergence_controls
from . import multi_field_solver_definition_commands
from . import multi_field_solver_global_controls
from . import multi_field_solver_interface_mapping
from . import multi_field_solver_load_transfer
from . import multi_field_solver_time_controls
from . import nonlinear_options
from . import ocean
from . import radiosity
from . import rezoning
from . import solid_body_loads
from . import solid_constraints
from . import solid_forces
from . import solid_surface_loads
from . import solution_status
from . import spectrum_options



def autots(self, key="", **kwargs):
    """Specifies whether to use automatic time stepping or load stepping.

    APDL Command: AUTOTS

    Parameters
    ----------
    key
        Automatic time stepping key:

        OFF - Do not use automatic time stepping.

        ON - Use automatic time stepping (default).

        AUTO - The program determines whether to use automatic time stepping (used by
               Workbench).

    Notes
    -----
    Specifies whether to use automatic time stepping (or load stepping)
    over this load step. If Key = ON, both time step prediction and time
    step bisection will be used.

    You cannot use automatic time stepping [AUTOTS], line search [LNSRCH],
    or the DOF solution predictor [PRED] with the arc-length method
    [ARCLEN, ARCTRM]. If you activate the arc-length method after you set
    AUTOTS, LNSRCH, or PRED, a warning message appears. If you choose to
    proceed with the arc-length method, the program disables your automatic
    time stepping, line search, and DOF predictor settings, and the time
    step size is controlled by the arc-length method internally.

    This command is also valid in PREP7.
    """
    command = f"AUTOTS,{key}"
    return self.run(command, **kwargs)


def campbell(self, action="", **kwargs):
    """Prepares the result file for a subsequent Campbell diagram of a

    APDL Command: CAMPBELL
    prestressed structure.

    Parameters
    ----------
    action
        Campbell action:

        NONE - Do not prepare the result file. This option is the default behavior.

        RSTP - Prepare the result file (Jobname.RSTP) for a subsequent Campbell diagram of a
               prestressed structure.

    Notes
    -----
    For an analysis involving a prestressed structure, the CAMPBELL command
    specifies whether or not to prepare the result file to support  a
    Campbell diagram analysis (PRCAMP or PLCAMP).

    To prestress a structure, the program performs a static solution before
    the linear perturbation modal solution.

    The CAMPBELL command requires that modal and static analyses be
    performed alternately. It works only when the number of static analyses
    is the same as the number of modal analyses. Any number of analyses can
    be performed, but the same number of each (static and modal) is
    expected. The modal solutions are appended in the results file
    (Jobname.RSTP).

    For an example of PLCAMP command usage, see Example Campbell Diagram
    Analysis in the Advanced Analysis Guide.
    """
    command = f"CAMPBELL,{action}"
    return self.run(command, **kwargs)


def cecmod(self, neqn="", const="", **kwargs):
    """Modifies the constant term of a constraint equation during solution.

    APDL Command: CECMOD

    Parameters
    ----------
    neqn
        Reference number of constraint equation.

    const
        New value of the constant term of equation.

    Notes
    -----
    Other terms of the constraint equation cannot be changed during the
    solution phase, but must be defined or changed within PREP7 prior to
    the solution.  See the CE command for details.

    This command is also valid in PREP7.
    """
    command = f"CECMOD,{neqn},{const}"
    return self.run(command, **kwargs)


def deltim(self, dtime="", dtmin="", dtmax="", carry="", **kwargs):
    """Specifies the time step sizes to be used for the current load step.

    APDL Command: DELTIM

    Parameters
    ----------
    dtime
        Time step size for this step. If automatic time stepping is used
        (AUTOTS), DTIME is the starting time substep.

    dtmin
        Minimum time step (if automatic time stepping is used). The program
        automatically determines the default based on the physics of the
        model.

    dtmax
        Maximum time step (if automatic time stepping is used). The program
        automatically determines the default based on the physics of the
        model.

    carry
        Time step carry over key:

        OFF - Use DTIME as time step at start of each load step.

        ON - Use final time step from previous load step as the starting time step (if
             automatic time stepping is used).

    Notes
    -----
    See NSUBST for an alternative input.

    Use consistent values for DTIME and TIME (TIME). For example, using 0.9
    for DTIME and 1.0 for TIME results in one time step because 1.0 (TIME)
    is divisible by .9 (DTIME) at most once. If you intend to load in 10
    increments over a time span of 1.0, use 0.1 for DTIME and 1.0 for TIME.

    The program calculates the initial incremental time so that (EndingTime
    - StartingTime)/DTIME is an integer, which may affect the initial
    incremental time that you specify. For example, if the starting time is
    0, the ending time is 1, and the initial incremental time is 0.4, the
    program rounds to the nearest integer and adjusts the time to 0.33333.

    For solution efficiency, specify values for all fields of this command.

    This command is also valid in PREP7.
    """
    command = f"DELTIM,{dtime},{dtmin},{dtmax},{carry}"
    return self.run(command, **kwargs)


def expsol(self, lstep="", sbstep="", timfrq="", elcalc="", **kwargs):
    """Specifies the solution to be expanded for mode-superposition analyses

    APDL Command: EXPSOL
    or substructure analyses.

    Parameters
    ----------
    lstep, sbstep
        Expand the solution identified as load step LSTEP and substep
        SBSTEP.

    timfrq
        As an alternative to LSTEP and SBSTEP, expand the solution at, or
        nearest to, the time value TIMFRQ (for ANTYPE,TRANS or
        ANTYPE,SUBSTR) or frequency value TIMFRQ (for ANTYPE,HARMIC).
        LSTEP and SBSTEP should be blank.

    elcalc
        Element calculation key:

        YES - Calculate element results, nodal loads, and reaction loads.

        NO - Do not calculate these items.

    Notes
    -----
    Specifies the solution to be expanded from analyses that use the mode-
    superposition method (ANTYPE,HARMIC or TRANS) or substructuring
    (ANTYPE,SUBSTR). Use the NUMEXP command to expand a group of solutions.

    The resulting results file will maintain the same load step, substep,
    and time (or frequency) values as the requested solution to be
    expanded.

    This command is also valid in PREP7.
    """
    command = f"EXPSOL,{lstep},{sbstep},{timfrq},{elcalc}"
    return self.run(command, **kwargs)


def kbc(self, key="", **kwargs):
    """Specifies ramped or stepped loading within a load step.

    APDL Command: KBC

    Parameters
    ----------
    key
        Ramping key:

        0 - Loads are linearly interpolated (ramped) for each substep from the values of
            the previous load step to the values of this load step. This is
            the default value.

        1 - Loads are step changed (stepped) at the first substep of this load step to the
            values of this load step (i.e., the same values are used for
            all substeps).  Useful for rate-dependent behavior (e.g.,
            creep, viscoplasticity, etc.) or transient load steps only.

    Notes
    -----
    Specifies whether loads applied to intermediate substeps within the
    load step are to be stepped or ramped. Used only if DTIME on the DELTIM
    command is less than the time span or, conversely, if NSBSTP on the
    NSUBST command is greater than one. Flags (FSI, MXWF, MVDI, etc.) are
    always stepped.

    Changing the ramping KEY (i.e., switching between ramped and stepped
    boundary conditions) between load steps is not recommended.

    For ramped loading (KBC,0), when a load is applied for the first time,
    it is interpolated from zero to the value of the current load step, and
    not from the initial condition or value of the degree of freedom from
    the previous load step.

    Spatially varying tabular loads or boundary conditions do not support
    direct ramping or stepping options and, instead, apply their full
    values according to the supplied tabular functions regardless of the
    KBC setting.

    For a static or harmonic cyclic symmetry analysis, any load that varies
    by sector (CYCOPT,LDSECT) is tabular and is applied as a step change,
    regardless of the KBC setting; however, any non-tabular loads in the
    same analysis are ramped or stepped according to the KBC setting.

    Irrespective of the KBC setting, loads are usually step-removed. See
    Stepping or Ramping Loads in the Basic Analysis Guide for more
    information.

    It is sometimes difficult to obtain successful convergence with stepped
    loading in a nonlinear transient problem. If divergence is encountered,
    determine if stepped loading was used by default, then determine if it
    is appropriate for the analysis.

    This command is also valid in PREP7.
    """
    command = f"KBC,{key}"
    return self.run(command, **kwargs)


def kuse(self, key="", **kwargs):
    """Specifies whether or not to reuse the factorized matrix.

    APDL Command: KUSE

    Parameters
    ----------
    key
        Reuse key:

        0  - Program decides whether or not to reuse the previous factorized stiffness
             matrix.

        1  - Force the previous factorized stiffness matrix to be reused.  Used mainly in a
             restart.  Forcing reuse of the matrix is a nonstandard use of
             the program, and should be done with caution.  For instance,
             using this option and changing the number of elements, or the
             number or type of degrees of freedom, may cause an abort.

        -1  - All element matrices are reformed and are used to reform a new factorized
              stiffness matrix.

    Notes
    -----
    Overrides the program logic to determine whether or not to reuse the
    previous factorized stiffness matrix for each substep of this load
    step.  Applies only to static or full transient analyses and to full
    harmonic analyses if the frequency is not changed for continuing
    loadsteps. For full harmonic analyses, only KEY = 1 or KEY = 0 is
    valid.

    This command is also valid in PREP7.
    """
    command = f"KUSE,{key}"
    return self.run(command, **kwargs)


def magopt(self, value="", **kwargs):
    """Specifies options for a 3-D magnetostatic field analysis.

    APDL Command: MAGOPT

    Parameters
    ----------
    value
        Option key:

        0 - Calculate a complete H field solution in the entire domain using a single
            (reduced) potential.

        Caution:When used in problems with both current sources and iron regions, errors may result due to numerical cancellation. - 1

        Calculate and store a preliminary H field in "iron" regions (μr ≠ 1).  Requires flux-parallel boundary conditions to be specified on exterior iron boundaries.  Used in conjunction with subsequent solutions with VALUE = 2 followed by VALUE = 3.  Applicable to multiply-connected iron domain problems. - 2

        Calculate and store a preliminary H field in "air" regions (μr = 1).  The air-iron interface is appropriately treated internally by the program.  Used in conjunction with a subsequent solution with VALUE = 3.  Applicable to singly-connected iron domain problems (with subsequent solution with VALUE = 3) or to multiply-connected iron domain problems (when preceded by a solution with VALUE = 1 and followed by a solution with VALUE = 3). - 3

    Notes
    -----
    Specifies the solution sequence options for a 3-D magnetostatic field
    analysis using a scalar potential (MAG).  The solution sequence is
    determined by the nature of the problem.

    You cannot use constraint equations with Value = 1.

    This command is also valid in PREP7.

    Distributed ANSYS Restriction: The MAGOPT,3 option is not supported in
    Distributed ANSYS when the following contact elements are present in
    the model: CONTA173, CONTA174, CONTA175, CONTA176, or CONTA177.
    """
    command = f"MAGOPT,{value}"
    return self.run(command, **kwargs)


def magsolv(self, opt="", nramp="", cnvcsg="", cnvflux="", neqit="",
            biot="", cnvtol="", **kwargs):
    """Specifies magnetic solution options and initiates the solution.

    APDL Command: MAGSOLV

    Parameters
    ----------
    opt
        Static magnetic solution option:

        0 - Vector potential (MVP) or edge formulation  (default).

        1 - Combined vector potential and reduced scalar potential (MVP-RSP).

        2 - Reduced scalar potential (RSP).

        3 - Difference scalar potential (DSP).

        4 - General scalar potential (GSP).

    nramp
        Number of ramped substeps for the first load step of a nonlinear
        MVP or MVP-RSP solution.  Defaults to 3.  If NRAMP = -1, ignore the
        ramped load step entirely.NRAMP is ignored for linear
        magnetostatics.

    cnvcsg
        Tolerance value on the program-calculated reference value for the
        magnetic current-segment convergence.  Used for the MVP, the MVP-
        RSP, and the edge formulation solution options (OPT = 0 and 1).
        Defaults to 0.001.

    cnvflux
        Tolerance value on the program-calculated reference value for the
        magnetic flux convergence.  Used for all scalar potential solution
        options (OPT = 2, 3, 4).  Defaults to 0.001.

    neqit
        Maximum number of equilibrium iterations per load step.  Defaults
        to 25.

    biot
        Option to force execution of a Biot-Savart integral solution
        [BIOT,NEW] for the scalar potential options.  Required if multiple
        load steps are being performed with different current source
        primitives (SOURC36 elements).

        0 - Do not force execution of Biot-Savart calculation (default); Biot-Savart is
            automatically calculated only for the first solution.

        1 - Force execution of Biot-Savart calculation.

    cnvtol
        Sets the convergence tolerance for AMPS reaction. Defaults to 1e-3.

    Notes
    -----
    MAGSOLV invokes an ANSYS macro which specifies magnetic solution
    options and initiates the solution.  The macro is applicable to any
    ANSYS magnetostatic analysis using the magnetic vector potential (MVP),
    reduced scalar potential (RSP), difference scalar potential (DSP),
    general scalar potential (GSP), or combined MVP-RSP formulation
    options.  Results are only stored for the final converged solution.
    (In POST1, issue *SET,LIST to identify the load step of solution
    results.)  The macro internally determines if a nonlinear analysis is
    required based on magnetic material properties.

    If you use the BIOT option and issue SAVE after solution or
    postprocessing, the Biot-Savart calculations are saved to the database,
    but will be overwritten upon normal exit from the program.  To save
    this data after issuing SAVE, use the /EXIT,NOSAVE command.  You can
    also issue the /EXIT,SOLU command to exit ANSYS and save all solution
    data, including the Biot-Savart calculations, in the database.
    Otherwise, when you issue RESUME, the Biot-Savart calculation will be
    lost (resulting in a zero solution).

    The MVP, MVP-RSP, and edge formulation options perform a two-load-step
    solution sequence.  The first load step ramps the applied loads over a
    prescribed number of substeps (NRAMP), and the second load step
    calculates the converged solution.  For linear problems, only a single
    load step solution is performed.  The ramped load step can be bypassed
    by setting NRAMP to -1.

    The RSP option solves in a single load step using the adaptive descent
    procedure.  The DSP option uses two load steps, and the GSP solution
    uses three load steps.

    The following analysis options and nonlinear options are controlled by
    this macro:  KBC, NEQIT, NSUBST, CNVTOL, NROPT, MAGOPT, and OUTRES.

    You cannot use constraint equations with OPT = 4.
    """
    command = f"MAGSOLV,{opt},{nramp},{cnvcsg},{cnvflux},{neqit},{biot},{cnvtol}"
    return self.run(command, **kwargs)


def mode(self, mode="", isym="", **kwargs):
    """Specifies the harmonic loading term for this load step.

    APDL Command: MODE

    Parameters
    ----------
    mode
        Number of harmonic waves around circumference for this harmonic
        loading term (defaults to 0).

    isym
        Symmetry condition for this harmonic loading term (not used when
        MODE = 0):

        1 - Symmetric (UX, UY, ROTZ, TEMP use cosine terms; UZ uses sine term) (default).

        -1 - Antisymmetric (UX, UY, ROTZ, TEMP use sine terms; UZ uses cosine term).

    Notes
    -----
    Used with axisymmetric elements having nonaxisymmetric loading
    capability (for example, PLANE25, SHELL61, etc.).  For analysis types
    ANTYPE,MODAL, HARMIC, TRANS, and SUBSTR, the term must be defined in
    the first load step and may not be changed in succeeding load steps.

    This command is also valid in PREP7.
    """
    command = f"MODE,{mode},{isym}"
    return self.run(command, **kwargs)


def nsubst(self, nsbstp="", nsbmx="", nsbmn="", carry="", **kwargs):
    """Specifies the number of substeps to be taken this load step.

    APDL Command: NSUBST

    Parameters
    ----------
    nsbstp
        Number of substeps to be used for this load step (i.e., the time
        step size or frequency increment). If automatic time stepping is
        used (AUTOTS), NSBSTP defines the size of the first substep.

    nsbmx
        Maximum number of substeps to be taken (i.e., the minimum time step
        size) if automatic time stepping is used. The program automatically
        determines the default based on the physics of the model.

    nsbmn
        Minimum number of substeps to be taken (i.e., the maximum time step
        size) if automatic time stepping is used. The program automatically
        determines the default based on the physics of the model.

    carry
        Time step carryover key (program-determined default depending on
        the problem physics):

        OFF - Use NSBSTP to define time step at start of each load step.

        ON - Use final time step from previous load step as the starting time step (if
             automatic time stepping is used).

    Notes
    -----
    See DELTIM for an alternative input.  It is recommended that all fields
    of this command be specified for solution efficiency and robustness.

    When the arc-length method is active (ARCLEN command), the NSBMX and
    NSBMN arguments are ignored.

    This command is also valid in PREP7.
    """
    command = f"NSUBST,{nsbstp},{nsbmx},{nsbmn},{carry}"
    return self.run(command, **kwargs)


def numexp(self, num="", begrng="", endrng="", elcalc="", **kwargs):
    """Specifies solutions to be expanded from mode-superposition analyses or

    APDL Command: NUMEXP
    substructure analyses.

    Parameters
    ----------
    num
        The number of solutions to expand. This value is required.

        Num - Number of solutions to expand.

        ALL - Expand all substeps between BEGRNG and ENDRNG (provided that ENDRNG > 0). If
              BEGRNG and ENDRNG have no specified values, this option
              expands all substeps of all load steps.

    begrng, endrng
        Beginning and ending time (or frequency) range for expanded
        solutions. The default is 0 for both values.

    elcalc
        The element-calculation key:

        YES - Calculate element results, nodal loads, and reaction loads. This value is the
              default.

        NO - Do not calculate these items.

    Notes
    -----
    Specifies a range of solutions to be expanded from analyses that use
    mode-superposition methods (ANTYPE,HARMIC or TRANS) or substructuring
    (ANTYPE,SUBSTR).

    For ANTYPE,TRANS, NUM, evenly spaced solutions are expanded between
    time BEGRNG and time ENDRNG.

    For ANTYPE,HARMIC, NUM, evenly spaced solutions are expanded between
    frequency BEGRNG and frequency ENDRNG.

    The first expansion in all cases is done at the first point beyond
    BEGRNG (that is, at BEGRNG + (ENDRNG - BEGRNG) / NUM)).

    The resulting results file will maintain the same load step, substep,
    and time (or frequency) values as the use pass.

    For a single expansion of a solution, or for multiple expansions when
    the solutions are not evenly spaced (such as in a mode-superposition
    harmonic analysis with the cluster option), ANSYS, Inc. recommends
    issuing one or more EXPSOL commands.

    The NUMEXP command is invalid in these cases:

    In a substructing analysis (ANTYPE,SUBST) when a factorized matrix file
    (the .LN22 file generated by the sparse solver) does not exist, causing
    ANSYS to employ the full-resolve method.

    If the full-resolve option is selected using the SEOPT command.

    In both situations, use the EXPSOL command to perform a single
    expansion for each solution desired.

    This command is also valid in PREP7.
    """
    command = f"NUMEXP,{num},{begrng},{endrng},{elcalc}"
    return self.run(command, **kwargs)


def time(self, time="", **kwargs):
    """Sets the time for a load step.

    APDL Command: TIME

    Parameters
    ----------
    time
        Time at the end of the load step.

    Notes
    -----
    Associates the boundary conditions at the end of the load step with a
    particular TIME value.

    TIME must be a positive, nonzero, monotonically increasing quantity
    that "tracks" the input history.  Units of time should be consistent
    with those used elsewhere (for properties, creep equations, etc.).

    Typically, for the first load step TIME defaults to 1. However, for the
    first load step of a mode-superposition transient analysis
    (ANTYPE,TRANS and TRNOPT,MSUP), the TIME command is ignored and a
    static solution is performed at TIME = 0.

    For a full transient analyses, the command's default behavior does not
    apply. You must specify a time for each load step and it must be
    greater than the time at the end of the prior load step.

    TIME does not apply to modal (ANTYPE,MODAL), harmonic (ANTYPE,HARMIC),
    or substructure (ANTYPE,SUBSTR) analyses.

    This command is also valid in PREP7.
    """
    command = f"TIME,{time}"
    return self.run(command, **kwargs)


def tref(self, tref="", **kwargs):
    """Defines the reference temperature for the thermal strain calculations.

    APDL Command: TREF

    Parameters
    ----------
    tref
        Reference temperature for thermal expansion.

    Notes
    -----
    Defines the reference temperature for the thermal strain calculations
    in structural analyses and explicit dynamic analyses.  Thermal strains
    are given by : α *(T-TREF), where α is the coefficient of thermal
    expansion (for more on this see the Mechanical APDL Theory Reference).
    Input the strain via ALPX, ALPY, ALPZ (the secant or mean coefficient
    value), or CTEX, CTEY, CTEZ (the instantaneous coefficient value), or
    the thermal strain value (THSX, THSY, THSZ). T is the element
    temperature. If α is temperature-dependent, TREF should be in the range
    of temperatures you define using the MPTEMP command.

    Reference temperatures may also be input per material by specifying a
    value on the MP material property command:

    MP,REFT,MAT,C0.

    Only a constant (non-temperature-dependent) value is valid. The value
    input on the TREF command applies to all materials not having a
    specified material property definition.

    To convert temperature-dependent secant coefficients of thermal
    expansion (SCTE) data (properties ALPX, ALPY, ALPZ) from the definition
    temperature to the reference temperature defined via a TREF (or
    MP,REFT) command, issue the MPAMOD command.

    This command is also valid in PREP7.
    """
    command = f"TREF,{tref}"
    return self.run(command, **kwargs)


def tsres(self, array="", **kwargs):
    """Defines an array of key times at which the time-stepping strategy

    APDL Command: TSRES
    changes.

    Parameters
    ----------
    array
        Identifies an Nx1x1 array parameter containing the key times at
        which the heat transfer time-stepping strategy changes (the time
        step is reset to the initial time step based on DELTIM or NSUBST
        settings).  The array name must be enclosed by % signs (e.g.,
        %array%).  See *DIM for more information on array parameters.

    Notes
    -----
    Time values in the array parameter must be in ascending order and must
    not exceed the time at the end of the load step as defined on the TIME
    command. The time increment between time points in the array list must
    be larger than the initial time step defined on the DELTIM or NSUBST
    command.  Time values must also fall between the beginning and ending
    time values of the load step.  For multiple load step problems, you
    must either change the parameter values to fall between the beginning
    and ending time values of the load step or reissue the command with a
    new array parameter.  To clear the array parameter specification, issue
    TSRES,ERASE.  Results can be output at the requested time points if the
    array or time values in the array are also specified in the OUTRES
    command using FREQ=%array%.  Use this command to reset the time-
    stepping strategy within a load step. You may need to reset the time-
    stepping strategy when using tabular time-varying boundary conditions.

    See Steady-State Thermal Analysis of the Thermal Analysis Guide  for
    more information on applying boundary conditions via tabular input.
    See Transient Thermal Analysis of the Thermal Analysis Guide for more
    information on defining the key time array.
    """
    command = f"TSRES,{array}"
    return self.run(command, **kwargs)


def upcoord(self, factor="", key="", **kwargs):
    """Modifies the coordinates of the active set of nodes, based on the

    APDL Command: UPCOORD
    current displacements.

    Parameters
    ----------
    factor
        Scale factor for displacements being added to nodal coordinates.
        If FACTOR = 1.0, the full displacement value will be added to each
        node, 0.5, half the displacement value will be added, etc.  If
        FACTOR = -1, the full displacement value will be subtracted from
        each node, etc.

    key
        Key for zeroing displacements in the database:

        OFF - Do not zero the displacements (default).

        ON - Zero the displacements.

    Notes
    -----
    The UPCOORD command uses displacements stored in the ANSYS database,
    and not those contained within the results file, Jobname.RST.  Nodal
    coordinates are updated each time the command is issued.  After
    updating, both the nodal displacements and rotations are set to zero if
    Key = ON.

    For structural solutions with an updated mesh, unless the coefficient
    matrix is otherwise reformed (e.g., a new analysis or NLGEOM,ON) it
    should first be reformed by issuing a KUSE,-1 command.

    UPCOORD should not be issued between load steps in structural analysis.

    For a multiphysics simulation where a CFD or electromagnetic field is
    being coupled to a structure undergoing large displacements, all (or a
    portion) of the surrounding field mesh may take part in the structural
    solution to "move" with the displacing structure.  You can use the
    UPCOORD command with a suitable FACTOR to update the coordinates of the
    nodes using the newly computed displacements.  The mesh will now
    conform with the displaced structure for subsequent field solutions.
    However, the mesh should always be restored to its original location by
    using an UPCOORD,FACTOR command before performing any subsequent
    structural solutions.  This is true for both repeated linear solutions,
    and for nonlinear restarts. (All saved displacements are relative to
    the original mesh location.)

    This command is not intended to replace either the large displacement
    or birth and death logic.

    This command is also valid in PREP7.
    """
    command = f"UPCOORD,{factor},{key}"
    return self.run(command, **kwargs)


def usrcal(self, rnam1="", rnam2="", rnam3="", rnam4="", rnam5="",
           rnam6="", rnam7="", rnam8="", rnam9="", **kwargs):
    """Allows user-solution subroutines to be activated or deactivated.

    APDL Command: USRCAL

    Parameters
    ----------
    rnam1, rnam2, rnam3, . . . , rnam9
        User-defined solution subroutine names to be activated.  Up to nine
        may be defined on one command or multiple commands may be used.  If
        Rnam1 = ALL, activate all valid user subroutines.   If Rnam1 =
        NONE, deactivate all valid user subroutines.  All characters are
        required:

        USREFL - Allows user defined scalar field (body force) loads.

        USERCV - Allows user defined convection (surface) loads.

        USERPR - Allows user defined pressure (surface) loads.

        USERFX - Allows user-defined heat flux (surface) loads.

        USERCH - Allows user-defined charge density (surface) loads.

        USERFD - Computes the complex load vector for the frequency domain logic.

        USEROU - Allows user supplied element output.

        USERMC - Allows user control of the hygrothermal growth).

        USOLBEG - Allows user access before each solution.

        ULDBEG - Allows user access before each load step.

        USSBEG - Allows user access before each substep.

        UITBEG - Allows user access before each equilibrium iteration.

        UITFIN - Allows user access after each equilibrium iteration.

        USSFIN - Allows user access after each substep.

        ULDFIN - Allows user access after each load step.

        USOLFIN - Allows user access after each solution.

        UANBEG - Allows user access at start of run.

        UANFIN - Allows user access at end of run.

        UELMATX - Allows user access to element matrices and load vectors.

        UTIMEINC - Allows a user-defined time step, overriding the program-determined time step.

        UCNVRG - Allows user-defined convergence checking, overriding the program-determined
                 convergence.

    Notes
    -----
    Allows certain user-solution subroutines to be activated or deactivated
    (system-dependent).  This command only affects the subroutines named.
    Other user subroutines (such as user elements, user creep, etc.) have
    their own activation controls described with the feature.

    The routines are commented and should be listed after performing a
    custom installation from the distribution media for more details.  See
    also the Advanced Analysis Guide for a general description of user-
    programmable features.

    Users must have system permission, system access, and knowledge to
    write, compile, and link the appropriate subroutines into the program
    at the site where it is to be run.  All routines should be written in
    FORTRAN. (For more information on FORTRAN compilers please refer to
    either the ANSYS, Inc. Windows Installation Guide or the ANSYS, Inc.
    Linux Installation Guide for details specific to your platform or
    operating system.) Issue USRCAL,STAT to list the status of these user
    subroutines.  Since a user-programmed subroutine is a nonstandard use
    of the program, the verification of any ANSYS run incorporating these
    commands is entirely up to the user.  In any contact with ANSYS
    customer support regarding the performance of a custom version of the
    ANSYS program, you should explicitly state that a user programmable
    feature has been used.

    This command is also valid in PREP7.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"USRCAL,{rnam1},{rnam2},{rnam3},{rnam4},{rnam5},{rnam6},{rnam7},{rnam8},{rnam9}"
    return self.run(command, **kwargs)


def wrfull(self, ldstep="", **kwargs):
    """Stops solution after assembling global matrices.

    APDL Command: WRFULL

    Parameters
    ----------
    ldstep
        Specify action to take:

        OFF or 0 - Turn off feature (default)

        N - Turn on feature and set it to stop after assembling the global matrices and
            writing the .FULL file for load step N.

    Notes
    -----
    This command is used in conjunction with the SOLVE command to generate
    the assembled matrix file (.FULL file) only.  The element matrices are
    assembled into the relevant global matrices for the particular analysis
    being performed and the .FULL file is written.  Equation solution and
    the output of data to the results file are skipped.  To dump the
    matrices written on the .FULL file into Harwell-Boeing format, use the
    HBMAT command in /AUX2. To copy the matrices to a postscript format
    that can be viewed graphically, use the PSMAT command.

    To use the LSSOLVE macro with this command, you may need to modify the
    LSSOLVE macro to properly stop at the load step of interest.

    This command only valid for linear static, full harmonic, and full
    transient analyses when the sparse direct solver is selected.  This
    command is also valid for buckling or modal analyses with any mode
    extraction method.  This command is not valid for nonlinear analyses.
    It is not supported in a linear perturbation analysis.

    In general, the assembled matrix file .FULL contains stiffness, mass,
    and damping matrices. However, the availability of the matrices depends
    on the analysis type chosen when the file is written.
    """
    command = f"WRFULL,{ldstep}"
    return self.run(command, **kwargs)



def cfclos(self, **kwargs):
    """Closes the "command" file.

    APDL Command: *CFCLOS

    Notes
    -----
    This command is valid in any processor.
    """
    command = f"*CFCLOS,"
    return self.run(command, **kwargs)


def cfopen(self, fname="", ext="", loc="", **kwargs):
    """Opens a "command" file.

    APDL Command: *CFOPEN

    .. warning::
       This command must be run using ``mapdl.non_interactive``

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum,
        including the characters needed for the directory path).
        An unspecified directory path defaults to the working
        directory; in this case, you can use all 248 characters
        for the file name.

    ext
        Filename extension (eight-character maximum).

    loc
        Determines whether existing file will be overwritten or appended:

        (blank) : The existing file will be overwritten.
        APPEND :  The file will be appended to the existing file.

    Notes
    -----
    Data processed with the *VWRITE command will also be written to this
    file if the file is open when the *VWRITE command is issued.

    This command is valid in any processor.
    """
    return self.run(f"*CFOPEN,{fname},{ext},,{loc}", **kwargs)


def cfwrite(self, command="", **kwargs):
    """Writes an ANSYS command (or similar string) to a "command" file.

    APDL Command: *CFWRITE

    Parameters
    ----------
    command
        Command or string to be written.  The standard command form of a
        label followed by arguments separated by commas is assumed.
        Command may be a parameter assignment (e.g.,  *CFWRITE, A = 5).

    Notes
    -----
    Writes an ANSYS command (or similar string) to the file opened with
    *CFOPEN.  The Command string is not executed (except that numeric and
    character parameter substitution and operations (with embedded *, /, >,$
    etc. characters) are performed before writing).  When used with *GET
    results and parameter substitution, an ANSYS command can be created
    from results and then read back into the ANSYS program (or used
    elsewhere).  For example, if the command *CFWRITE,BF,NNUM,TEMP,TVAL is
    used in a do-loop, where TVAL is a parameter value returned from the
    *GET operation and NNUM is a specified or returned parameter value, a
    series of BF  commands, with numerical values substituted for the two
    parameters, will be written.  To create a file without parameter
    substitution, use *CREATE.

    This command is valid in any processor.
    """
    command = f"*CFWRITE,{command}"
    return self.run(command, **kwargs)


def create(self, fname="", ext="", **kwargs):
    """Opens (creates) a macro file.

    APDL Command: *CREATE

    .. warning::
       This command must be run using ``mapdl.non_interactive``

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    See the *USE command for a discussion of macros.  All commands
    following the *CREATE command, up to the *END command, are written to
    the specified file without being executed.  An existing file of the
    same name, if any, will be overwritten.  Parameter values are not
    substituted for parameter names in the commands when the commands are
    written to the file.  Use *CFWRITE to create a file if this is desired.
    The resulting macro may be executed with a *USE command (which also
    allows parameters to be passed into the macro) or a /INPUT command
    (which does not allow parameters to be passed in).  Several macros may
    be stacked into a library file [*ULIB]. You cannot use *CREATE within a
    DO loop.

    This command is valid in any processor.
    """
    return self.run(f"*CREATE,{fname},{ext}", **kwargs)


def dflab(self, dof="", displab="", forcelab="", **kwargs):
    """Changes degree-of-freedom labels for user custom elements.

    APDL Command: /DFLAB

    Parameters
    ----------
    dof
        Number between 1 and 32 indicating which degree of freedom is to
        have its labels changed. For a list of these quantities, see the
        degree-of-freedom table in the echprm.inc file. The first few
        quantities follow:

    displab
        New label (four-character maximum) for the displacement label. The
        prior label is no longer valid.

    forcelab
        New label (four-character maximum) for the force label for this
        degree of freedom. The prior label is no longer valid.

    Notes
    -----
    The /DFLAB command is rarely used. Use it if you are writing a custom
    element and want to use degrees of freedom that are not part of the
    standard element set.
    """
    command = f"/DFLAB,{dof},{displab},{forcelab}"
    return self.run(command, **kwargs)


def end(self, **kwargs):
    """Closes a macro file.

    APDL Command: *END

    Notes
    -----
    Closes a file opened with *CREATE. The *END command is an 8-character
    command (to differentiate it from *ENDIF).   If you add commented text
    on that same line but do not allow enough spaces between *END and the
    "!" that indicates the comment text, the *END will attempt to interpret
    the "!" as the 8th character and will fail.

    This command is valid in any processor.
    """
    command = f"*END,"
    return self.run(command, **kwargs)


def mkdir(self, dir_="", **kwargs):
    """Creates a directory.

    APDL Command: /MKDIR

    Parameters
    ----------
    dir_
        The directory to create (248 characters maximum on Linux;
        233 on Windows). If no path is provided, it will be
        created in the current working directory. Must be a valid
        name (and path) for the system you are working on.

    Notes
    -----
    It is recommended to just use ``os.mkdir``

    Creates a directory on the computer ANSYS is currently running on.
    """
    command = f"/MKDIR,{dir_}"
    return self.run(command, **kwargs)


def msg(self, lab="", val1="", val2="", val3="", val4="", val5="", val6="",
        val7="", val8="", **kwargs):
    """Writes an output message via the ANSYS message subroutine.

    APDL Command: *MSG

    Parameters
    ----------
    lab
        Label for output and termination control:

        Writes the message with no heading (default). - Writes the message with a "NOTE" heading.

        Writes the message with a "WARNING" heading.  Also writes the message to the errors file, Jobname.ERR. - Writes the message with a "ERROR" heading and causes run termination (if batch)
                          at earliest "clean exit" point.  Also writes the
                          message to the errors file, Jobname.ERR.

        Writes the message with a "FATAL ERROR" heading and causes run termination immediately.  Also writes the message to the errors file, Jobname.ERR. - Writes the message with a "NOTE" heading and displays it in the message dialog
                          box.  This option is most useful in GUI mode.

    val1, val2, val3, . . . , val8
        Numeric or alphanumeric character values to be included in message.
        Values may be the results of parameter evaluations.  All numeric
        values are assumed to be double precision. The FORTRAN nearest
        integer (NINT) function is used to form integers for the %I
        specifier.

    Notes
    -----
    Allows writing an output message via the ANSYS message subroutine.
    Also allows run termination control.  This command is used only when
    contained in a prepared file read into the ANSYS program (i.e.,
    *USE,/INPUT, etc.).  A message format must immediately follow the *MSG
    command (on a separate line, without parentheses, as described below).

    The message format may be up to 80 characters long, consisting of text
    strings and predefined "data descriptors" between the strings where
    numeric or alphanumeric character data are to be inserted.  The normal
    descriptors are %I for integer data, %G for double precision data, %C
    for alphanumeric character data, and %/ for a line break.  The
    corresponding FORTRAN data descriptors are I9, 1PG16.9 and A8,
    respectively.  Each descriptor must be preceded by a blank.  There must
    be one data descriptor for each specified value (8 maximum) in the
    order of the specified values.

    Enhanced descriptions may also be used:

    Do not begin *MSG format lines with *IF, *ELSE , *ELSEIF , or *ENDIF .
    If the last nonblank character of the message format is an ampersand
    (&), a second line will also be read as a continuation of the format.
    Up to nine continuations (ten total lines) may be read.  If normal
    descriptions are used, then consecutive blanks are condensed into one
    blank upon output, and a period is appended.  Up to ten lines of output
    of 72 characters each may be produced (using the %/ descriptor).  Two
    examples follow.

    Here is an example of the *MSG command and a format to print a message
    with two integer values and one real value:

    The output line is:

    Here is an example illustrating multiline displays in GUI message
    windows:

    Note:: : The /UIS,MSGPOP command controls which messages are displayed
    in the message dialog box when the GUI is active.  All messages
    produced by the *MSG command are subject to the /UIS specification,
    with one exception,  If Lab = UI, the message will be displayed in the
    dialog box regardless of the /UIS specification.

    This command is valid in any processor.
    """
    command = f"*MSG,{lab},{val1},{val2},{val3},{val4},{val5},{val6},{val7},{val8}"
    return self.run(command, **kwargs)


def pmacro(self, **kwargs):
    """Specifies that macro contents be written to the session log file.

    APDL Command: /PMACRO

    Notes
    -----
    This command forces the contents of a macro or other input file to be
    written to Jobname.LOG.  It is valid only within a macro or input file,
    and should be placed at the top of the file.  /PMACRO should be
    included in any macro or input file that calls GUI functions.
    """
    command = f"/PMACRO,"
    return self.run(command, **kwargs)


def psearch(self, pname="", **kwargs):
    """Specifies a directory to be searched for "unknown command" macro files.

    APDL Command: /PSEARCH

    Parameters
    ----------
    pname
        Path name (64 characters maximum, and must include the final
        delimiter) of the middle directory to be searched.  Defaults to the
        user home directory.  If Pname = OFF, search only the ANSYS and
        current working directories.  If Pname = STAT, list the current
        middle directory and show the ANSYS_MACROLIB setting.

    Notes
    -----
    Specifies the pathname of a directory for file searches when reading
    "unknown command" macro files.  The search for the files is typically
    from the ANSYS directory, then from the user home directory, and then
    from the current working directory.  This command allows the middle
    directory searched to be other than the user home directory.

    This command is valid only at the Begin Level.
    """
    command = f"/PSEARCH,{pname}"
    return self.run(command, **kwargs)


def rmdir(self, dir_="", **kwargs):
    """Removes (deletes) a directory.

    APDL Command: /RMDIR

    Parameters
    ----------
    dir\_
        The directory to remove. If no path is provided, it will be assumed
        to be in the current working directory. All files in the directory
        are also removed.

    Notes
    -----
    Removes a directory on the computer ANSYS is currently running on. No
    warning or prompt is given, so use with extreme caution.
    """
    command = f"/RMDIR,{dir_}"
    return self.run(command, **kwargs)


def tee(self, label="", fname="", ext="", **kwargs):
    """Writes a list of commands to a specified file at the same time that the

    APDL Command: /TEE
    commands are being executed.

    Parameters
    ----------
    label
        Indicates how ANSYS is to interpret this /TEE command:

        Signals the beginning of the command text that is to be
        written to Fname. If Fname already exists, specifying NEW
        causes the contents of Fname to be overwritten. -
        Indicates that you want to append to Fname the command
        text that follows.

    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    You can use the /TEE command to record a macro to a specified file at
    the same time that the macro is being executed. It is similar to the
    Linux tee command.

    For more information about the /TEE command, see the Introducing APDL
    of the ANSYS Parametric Design Language Guide.

    The following example illustrates the use of the /TEE command. If you
    issue these commands:

    the content of myfile.mac is:

    This command is valid in any processor, but only during an interactive
    run.
    """
    command = f"/TEE,{label},{fname},{ext}"
    return self.run(command, **kwargs)


def ulib(self, fname="", ext="", **kwargs):
    """Identifies a macro library file.

    APDL Command: *ULIB

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    Identifies a macro library file for the *USE command.  A
    library of macros allows blocks of often used ANSYS commands
    to be stacked and executed from a single file.  The macro
    blocks must be enclosed within block identifier and terminator
    lines as shown in the example below.  If you want to add
    comment lines to a macro block, you may place them anywhere
    within the macro block.  (This includes placing them directly
    on the lines where the macro block identifier and the macro
    block terminator appear, as shown in the example.)  Do not
    place comment lines (or any other lines) outside of a macro
    block.

    The name of the macro library file is identified for reading
    on the *ULIB command.  The name of the macro block is
    identified on the *USE command.  The commands within the macro
    block are copied to a temporary file (of the macro block name)
    during the *USE operation and executed as if a macro file of
    that name had been created by the user.  The temporary file is
    deleted after it has been used.  Macro block names should be
    acceptable filenames (system dependent) and should not match
    user created macro file names, since the user macro file will
    be used first (if it exists) before the library file is
    searched.  Macro blocks may be stacked in any order.
    Branching [*GO or *IF] external to the macro block is not
    allowed.

    This command is valid in any processor.
    """
    command = f"*ULIB,{fname},{ext}"
    return self.run(command, **kwargs)


def use(self, name="", arg1="", arg2="", arg3="", arg4="", arg5="",
        arg6="", arg7="", arg8="", arg9="", ar10="", ar11="", ar12="",
        ar13="", ar14="", ag15="", ar16="", ar17="", ar18="", **kwargs):
    """Executes a macro file.

    APDL Command: *USE

    Parameters
    ----------
    name
        Name (32 characters maximum, beginning with a letter) identifying
        the macro file or a macro block on a macro library file.

    arg1, arg2, arg3, . . . , ar18
        Values passed into the file or block where the parameters ARG1
        through ARG9 and AR10 through AR18 are referenced.  Values may be
        numbers, alphanumeric character strings (up to 32 characters
        enclosed in single quotes), parameters (numeric or character) or
        parametric expressions.  See below for additional details.

    Notes
    -----
    Causes execution of a macro file called Name, or, if not found, a macro
    block "Name" on the macro library file [*ULIB].  Argument values
    (numeric or character) are passed into the file or block and
    substituted for local parameters ARG1, ARG2, ..., AR18.  The file Name
    may also be executed as an "unknown command" (i.e., without the *USE
    command name) as described below.

    A macro is a sequence of ANSYS commands (as many as needed) recorded in
    a file or in a macro block in a library file (specified with the *ULIB
    command).  The file or block is typically executed with the *USE
    command.  In addition to command, numerical and alphanumeric data, the
    macro may include parameters which will be assigned numerical or
    alphanumerical character values when the macro is used.  Use of the
    macro may be repeated (within a do-loop, for example) with the
    parameters incremented.  A macro is defined within a run by "enclosing"
    a sequence of data input commands between a *CREATE and a *END
    command.  The data input commands are passive (not executed) while
    being written to the macro file.  The macro file (without *CREATE and
    *END ) can also be created external to ANSYS.

    Up to 99 specially named scalar parameters called ARG1 to AR99 are
    locally available to each macro.  Note that the prefix for the first 9
    parameters is "ARG," while the prefix for the last 90 is "AR."  A local
    parameter is one which is not affected by, nor does it affect, other
    parameters, even those of the same name, which are used outside of the
    macro.  The only way a local parameter can affect, or be affected by,
    parameters outside the macro is if values are passed out of, or into,
    the macro by an argument list.  Parameters ARG1 through AR18 can have
    their values (numeric or character) passed via the argument list on the
    *USE command (ARG1 through AR19 can be passed as arguments on the
    "unknown command" macro).  Parameters AR19 through AR99 (AR20 through
    AR99 in the "unknown command" macro) are available solely for use
    within the macro; they cannot be passed via an argument list.  Local
    parameters are available to do-loops and to /INPUT files processed
    within the macro.  In addition to an ARG1--AR99 set for each macro,
    another ARG1--AR99 set is available external to all macros, local to
    "non-macro" space.

    A macro is exited after its last line is executed.  Macros may be
    nested (such as a *USE or an "unknown command" within a macro).  Each
    nested macro has its own set of 99 local parameters.  Only one set of
    local parameters can be active at a time and that is the set
    corresponding to the macro currently being executed or to the set
    external to all macros (if any).  When a nested macro completes
    execution, the previous set of local parameters once again becomes
    available.  Use *STATUS,ARGX to view current macro parameter values.

    An alternate way of executing a macro file is via the "unknown command"
    route.  If a command unknown to the ANSYS program is entered, a search
    for a file of that name (plus a .MAC suffix) is made.  If the file
    exists, it is executed, if not, the "unknown command" message is
    output.  Thus, users can write their own commands in terms of other
    ANSYS commands.  The procedure is similar to issuing the *USE command
    with the unknown command in the Name field.  For example, the command
    CMD,10,20,30 is internally similar to *USE,CMD,10,20,30.  The macro
    file named CMD.MAC will be executed with the three parameters.  The
    *USE macro description also applies to the "unknown command" macro,
    except that various directories are searched and a suffix (.MAC) is
    assumed.  Also, a macro library file is not searched.

    A three-level directory search for the "unknown command" macro file may
    be available (see the Operations Guide).  The search order may be: 1) a
    high-level system directory, 2) the login directory, and 3) the local
    (working) directory.  Use the /PSEARCH command to change the directory
    search path.  For an "unknown command" CMD, the first file named
    CMD.MAC found to exist in the search order will be executed.  The
    command may be input as upper or lower case, however, it is converted
    to upper case before the file name search occurs.  On systems that
    uniquely support both upper and lower case file names, the file with
    the matching lower case name will be used if it exists, otherwise, the
    file with the matching upper case name will be used. All macro files
    placed in the apdl directory must be upper case.

     Note, since undocumented commands exist in the ANSYS program, the user
    should issue the command intended for the macro file name to be sure
    the "unknown command" message is output in the processor where it's to
    be used.  If the macro is to be used in other processors, the other
    processors must also be checked.

    This command is valid in any processor.
    """
    command = f"*USE,{name},{arg1},{arg2},{arg3},{arg4},{arg5},{arg6},{arg7},{arg8},{arg9},{ar10},{ar11},{ar12},{ar13},{ar14},{ag15},{ar16},{ar17},{ar18}"
    with self.non_interactive:
        return self.run(command, **kwargs)



def wait(self, dtime="", **kwargs):
    """APDL Command: /WAIT

    Causes a delay before the reading of the next command.

    Parameters
    ----------
    dtime
        Time delay (in seconds). Maximum time delay is 59 seconds.

    Notes
    -----
    You should consider using ``time.sleep(dtime)``

    The command following the /WAIT will not be processed until the
    specified wait time increment has elapsed.  Useful when reading from a
    prepared input file to cause a pause, for example, after a display
    command so that the display can be reviewed for a period of time.
    Another "wait" feature is available via the *ASK command.

    This command is valid in any processor.
    """
    command = f"/WAIT,{dtime}"
    return self.run(command, **kwargs)



def bfint(self, fname1="", ext1="", fname2="", ext2="", kpos="", clab="",
          kshs="", tolout="", tolhgt="", **kwargs):
    """Activates the body force interpolation operation.

    APDL Command: BFINT

    Parameters
    ----------
    fname1
        File name and directory path (248 characters maximum,
        including directory) from which to read data for
        interpolation. If you do not specify a directory path, it
        will default to your working directory and you can use all
        248 characters for the file name.

    ext1
        Filename extension (eight-character maximum).

    fname2
        File name and directory path (248 characters maximum,
        including directory) to which BF commands are written. If
        you do not specify a directory path, it will default to
        your working directory and you can use all 248 characters
        for the file name.

    ext2
        Filename extension (eight-character maximum).

    kpos
        Position on Fname2 to write block of BF commands:

        0 - Beginning of file (overwrite existing file).

        1 - End of file (append to existing file).

    clab
        Label (8 characters maximum, including the colon) for this
        block of BF commands in Fname2.  This label is appended to
        the colon (:).  Defaults to BFn, where n is the cumulative
        iteration number for the data set currently in the
        database.

    kshs
        Shell-to-solid submodeling key:

        0 - Solid-to-solid or shell-to-shell submodel.

        1 - Shell-to-solid submodel.

    tolout
        Extrapolation tolerance about elements, based on a
        fraction of the element dimension. Submodel nodes outside
        the element by more than TOLOUT are not accepted as
        candidates for DOF extrapolation.  Defaults to 0.5 (50%).

    tolhgt
        Height tolerance above or below shell elements, in units
        of length.  Used only for shell-to-shell submodeling (KSHS
        = 0). Submodel nodes off the element surface by more than
        TOLHGT are not accepted as candidates for DOF
        interpolation or extrapolation. Defaults to 0.0001 times
        the maximum element dimension.

    Notes
    -----
    File Fname1 should contain a node list for which body forces are to be
    interpolated [NWRITE].  File Fname2 is created, and contains
    interpolated body forces written as a block of nodal BF commands.

    Body forces are interpolated from elements having TEMP as a valid body
    force or degree of freedom, and only the label TEMP is written on the
    nodal BF commands.  Interpolation is performed for all nodes on file
    Fname1 using the results data currently in the database. For layered
    elements, use the LAYER command to select the locations of the
    temperatures to be used for interpolation. Default locations are the
    bottom of the bottom layer and the top of the top layer.

    The block of BF commands begins with an identifying colon label command
    and ends with a /EOF command.  The colon label command is of the form
    :Clab, where Clab is described above.  Interpolation from multiple
    results sets can be performed by looping through the results file in a
    user-defined macro.  Additional blocks can be appended to Fname2 by
    using KPOS and unique colon labels.  A /INPUT command, with the
    appropriate colon label, may be used to read the block of commands.

    If the model has coincident (or very close) nodes, BFINT must be
    applied to each part of the model separately to ensure that the mapping
    of the nodes is correct.  For example, if nodes belonging to two
    adjacent parts linked by springs are coincident, the operation should
    be performed on each part of the model separately.
    """
    command = f"BFINT,{fname1},{ext1},,{fname2},{ext2},,{kpos},{clab},{kshs},{tolout},{tolhgt}"
    return self.run(command, **kwargs)


def cbdof(self, fname1="", ext1="", fname2="", ext2="", kpos="", clab="",
          kshs="", tolout="", tolhgt="", tolthk="", **kwargs):
    """Activates cut-boundary interpolation (for submodeling).

    APDL Command: CBDOF

    Parameters
    ----------
    fname1
        File name and directory path (248 characters maximum,
        including directory) from which to read boundary node
        data. If no specified directory path exists, the path
        defaults to your working directory and you can use all 248
        characters for the file name.

    ext1
        Filename extension (eight-character maximum).

    fname2
        File name and directory path (248 characters maximum,
        including directory) to which cut-boundary D commands are
        written. If no specified directory path exists, the path
        defaults to your working directory and you can use all 248
        characters for the file name.

    ext2
        Filename extension (eight-character maximum).

    kpos
        Position on Fname2 to write block of D commands:

        0 - Beginning of file (overwrite existing file).

        1 - End of file (append to existing file).

    clab
        Label (eight characters maximum, including the colon) for
        this block of D commands on Fname2.  his label is appended
        to the colon (:).  Defaults to CBn, where n is the
        cumulative iteration number for the data set currently in
        the database.  For imaginary data (see KIMG on the *SET
        command), Clab defaults to CIn.

    kshs
        Shell-to-solid submodeling key:

        0 - Solid-to-solid or shell-to-shell submodel.

        1 - Shell-to-solid submodel.

    tolout
        Extrapolation tolerance about elements, based on a
        fraction of the element dimension. Submodel nodes outside
        the element by more than TOLOUT are not accepted as
        candidates for DOF extrapolation.  Defaults to 0.5 (50
        percent).

    tolhgt
        Height tolerance above or below shell elements, in units
        of length.  Used only for shell-to-shell submodeling (KSHS
        = 0). Submodel nodes off the element surface by more than
        TOLHGT are not accepted as candidates for
        degree-of-freedom interpolation or extrapolation.
        Defaults to 0.0001 times the maximum element dimension.

    tolthk
        Height tolerance above or below shell elements, based on a
        fraction of the shell element thickness. Used only for
        shell-to-solid submodeling (KSHS = 1). Submodel nodes off
        the element surface by more than TOLTHK are not accepted
        as candidates for DOF interpolation or
        extrapolation. Defaults to 0.1 times the average shell
        thickness.

    Notes
    -----
    File Fname1 should contain a node list for which boundary
    conditions are to be interpolated (NWRITE).  File Fname2 is
    created to contain interpolated boundary conditions written as
    a block of D commands.

    Boundary conditions are written for the active
    degree-of-freedom set for the element from which interpolation
    is performed. Interpolation occurs on the selected set of
    elements.  The block of D commands begins with an identifying
    colon label and ends with a /EOF command.  The colon label is
    of the form :Clab (described above).

    Interpolation from multiple results sets can be performed by
    looping through the results file in a user-defined macro.
    Additional blocks can be appended to Fname2 by using KPOS and
    unique colon labels.  To read the block of commands, issue the
    /INPUT command with the appropriate colon label.

    If the model has coincident (or very close) nodes, the CBDOF
    must be applied to each part of the model separately to ensure
    that the mapping of the nodes is correct.  For example, if
    nodes belonging to two adjacent parts linked by springs are
    coincident, the operation should be performed on each part of
    the model separately.

    Resume the coarse model database at the beginning of the
    cut-boundary procedure. The database should have been saved
    after the first coarse model solution, as the number of nodes
    in the database and the results file must match, and internal
    nodes are sometimes created during the solution.

    Caution: Relaxing the TOLHGT or TOLTHK tolerances to allow
    submodel nodes to be "found" can produce poor submodel
    results.
    """
    command = f"CBDOF,{fname1},{ext1},,{fname2},{ext2},,{kpos},{clab},{kshs},{tolout},{tolhgt},{tolthk}"
    return self.run(command, **kwargs)


def cmsfile(self, option="", fname="", ext="", cmskey="", **kwargs):
    """Specifies a list of component mode synthesis (CMS) results files for

    APDL Command: CMSFILE
    plotting results on the assembly.

    Parameters
    ----------
    option
        Specifies the command operation:

        ADD - Add the specified component results file (Fname) to the list of files to plot.
              This option is the default.

        DELETE - Remove the specified component results file (Fname) from the list of files to
                 plot.

        LIST - List all specified component results files.

        CLEAR - Clear all previous files added.

        ALL - Add all component results (.rst) files from the working directory to the list
              of files to plot.

    fname
        The file name (with full directory path) of the component results
        file. The default file name is the Jobname (specified via the
        /FILNAME command).

    ext
        The file name (Fname) extension. The default extension is .rst.

    cmskey
        Valid only when adding a results file (Option = ADD or ALL), this
        key specifies whether or not to check the specified .rst file to
        determine if it was created via a CMS expansion pass:

        ON - Check (default).

        OFF - Do not check.

    Notes
    -----
    The CMSFILE command specifies the list of component mode synthesis
    (CMS) results files to include when plotting the mode shape of an
    assembly.

    During postprocessing (/POST1) of a CMS analysis, issue the CMSFILE
    command to point to component results files of interest. (You can issue
    the command as often as needed to include all or some of the component
    results files.) Issue the SET command to acquire the frequencies and
    mode shapes from substeps for all specified results files. Execute a
    plot (PLNSOL) or print (PRNSOL) operation to display the mode shape of
    the entire assembly.

    When you specify a results file to add to the plot list, the default
    behavior of the command (CmsKey = ON) is to first verify that the file
    is from a CMS analysis and that the frequencies of the result sets on
    the file match the frequencies on the first file in the list. If CmsKey
    = OFF, you can add any .rst file to the list of files to plot, even if
    the file was not expanded via a CMS expansion pass.

    If CmsKey = ON (default), output from the command appears as: ADD CMS
    FILE = filename.rst. : If CmsKey = OFF, output from the command appears
    as: ADD FILE = filename.rst.

    If Option = DELETE or CLEAR, you must clear the database (/CLEAR), then
    re-enter the postprocessor (/POST1) and issue a SET command for the
    change to take effect on subsequent plots.

    Clearing the database does not clear the list of files specified via
    the CMSFILE command. Specify Option = CLEAR to clear the list of files.
    """
    command = f"CMSFILE,{option},{fname},{ext},{cmskey}"
    return self.run(command, **kwargs)


def cyccalc(self, fileprefix="", fileformat="", **kwargs):
    """Calculates results from a cyclic harmonic mode-superposition analysis

    APDL Command: CYCCALC
    using the specifications defined by CYCSPEC.

    Parameters
    ----------
    fileprefix
        Each result table (corresponding to each CYCSPEC specification) is
        written to a file beginning with FilePrefix. If blank (default),
        the result tables are written to the output file.

    fileformat
        If FilePrefix is specified, then use FileFormat to specify the
        format of the file to be written:

        FORM - Formatted file (default)

        CSV - Comma-separated value file

    Notes
    -----
    CYCCALC loops through the specification given by CYCSPEC and computes
    the requested outputs. The outputs are given in a table format, with
    the rows corresponding to each frequency solution from the harmonic
    analysis, and the columns corresponding to each sector. The table
    entries are the maximum value of the specified quantity at the
    specified location in the sector. In addition, columns containing the
    maximum value at the frequency, the sector in which it occurs, and the
    node in the sector at which it occurs are output.

    If FilePrefix is specified, a file is created for each output table
    with the name FilePrefix_node_type.ext, where node is the node number
    or component name, type is the item/component requested, and the file
    extension .ext is either .txt or .csv, depending on FileFormat.

    A SET command must precede the CYCCALC command.

    The CYCCALC results are based on the currently active RSYS, SHELL,
    LAYER, and AVPRIN settings.
    """
    command = f"CYCCALC,{fileprefix},{fileformat}"
    return self.run(command, **kwargs)


def cycfiles(self, fnamerst="", extrst="", fnamerfrq="", extrfrq="",
             **kwargs):
    """Specifies the data files where results are to be found for a cyclic

    APDL Command: CYCFILES
    symmetry mode-superposition harmonic analysis.

    Parameters
    ----------
    fnamerst
        The file name and directory path of the results file from the
        cyclic modal solution. Defaults to Jobname.

    extrst
        File name extension for FnameRst. Defaults to rst.

    fnamerfrq
        The file name and directory path of the results file from the
        cyclic mode-superposition harmonic solution. Defaults to the value
        of the FnameRst argument.

    extrfrq
        File name extension for FnameRfrq. Defaults to rfrq.
    """
    command = f"CYCFILES,{fnamerst},{extrst},{fnamerfrq},{extrfrq}"
    return self.run(command, **kwargs)


def cycphase(self, type_="", option="", **kwargs):
    """Provides tools for determining minimum and maximum possible result

    APDL Command: CYCPHASE
    values from frequency couplets produced in a modal cyclic symmetry
    analysis.

    Parameters
    ----------
    type\_
        The type of operation requested:

        DISP - Calculate the maximum and minimum possible displacement at each node in the
               original sector model.  Store the values and the phase angle
               at which they occurred.

        STRESS - Calculate the maximum and minimum possible stresses at each node in the
                 original sector model.  Store the values and the phase
                 angle at which they occurred.

        STRAIN - Calculate the maximum and minimum possible strains at each node in the original
                 sector model.  Store the values and the phase angle at
                 which they occurred.

        ALL - Calculate the maximum and minimum possible displacement, stress and strain at
              each node in the original sector model.  Store the values and
              the phase angle at which they occurred.

        GET - Places the value of a MAX or MIN item into the _CYCVALUE parameter, the node
              for that value in the _CYCNODE parameter, and the phase angle
              for the value in the _CYCPHASE parameter.

        PUT - Put resulting sweep values for printing (via the PRNSOL command ) or plotting
              (via the PLNSOL command).

        LIST - List the current minimum/maximum displacement, stress and strain nodal values.

        STAT - Summarize the results of the last phase sweep.

        CLEAR - Clear phase-sweep information from the database.

    option
        If TYPE = DISP, STRAIN, STRESS or ALL, controls the sweep angle
        increment to use in the search:

        Angle - The sweep angle increment in degrees, greater than 0.1 and less than 10. The
                default is 1.

    Notes
    -----
    When you expand the results of a modal cyclic symmetry analysis (via
    the /CYCEXPAND or EXPAND command), ANSYS combines the real and
    imaginary results for a given nodal diameter, assuming no phase shift
    between them; however, the modal response can occur at any phase shift.

    CYCPHASE response results are valid only for the first cyclic sector.
    To obtain the response at any part of the expanded model, ANSYS, Inc.
    recommends using cyclic symmetry results expansion at the phase angle
    obtained via CYCPHASE.

    The phase angles returned by CYCPHASE contain the minimum and maximum
    values for USUM, SEQV and other scalar principal stress and strain
    quantities; however, they do not always return the true minimum and
    maximum values for directional quantities like UX or SX unless the
    values fall in the first sector.

    CYCPHASE does not consider midside node values when evaluating maximum
    and minimum values, which may affect DISPLAY quantities but no others.
    (Typically, ANSYS ignores midside node stresses and strains during
    postprocessing.)

    Issuing CYCPHASE,PUT clears the result values for midside nodes on high
    order elements; therefore, this option sets element faceting (/EFACET)
    to 1. The command reports that midside nodal values are set to zero and
    indicates that element faceting is set to 1.

    If the sweep values are available after issuing a CYCPHASE,PUT command,
    the PRNSOL or PLNSOL command will print or plot (respectively) the
    sweep values of structure displacement Ux, Uy, Uz, component
    stress/strain X, Y, Z, XY, YZ, ZX, principal stress/strain 1, 2, 3 and
    equivalent stress/strain EQV.  The vector sum of displacement (USUM)
    and stress/strain intensity (SINT) are not valid phase-sweep results.

    You can specify any coordinate system via the RSYS command for
    displaying or printing CYCPHASE results. However, after CYCPHASE
    results have been extracted, you cannot then transform them via the
    RSYS command. If you try to do so, ANSYS issues a warning message.

    The CYCPHASE command is valid in /POST1 and for cyclically symmetric
    models only.

    To learn more about analyzing a cyclically symmetric structure, see the
    Cyclic Symmetry Analysis Guide.
    """
    command = f"CYCPHASE,{type_},{option}"
    return self.run(command, **kwargs)


def cycspec(self, label="", node="", item="", comp="", **kwargs):
    """Defines the set of result items for a subsequent CYCCALC command in

    APDL Command: CYCSPEC
    postprocessing a cyclic harmonic mode-superposition analysis.

    Parameters
    ----------
    label
        One of the following labels:

        ADD - Adds a new specification to the set (default). The maximum number of
              specifications that can be defined is 50.

        LIST - Lists the current set of specifications. Node, Item, Comp are ignored.

        ERASE - Erases the current set of specifications. Node, Item, Comp are ignored.

        DELETE - Deletes an existing specification. Item, Comp are ignored.

    node
        The node at which to evaluate the results. If Node is a nodal
        component, then all nodes in the component are included. All
        sectors containing this node (or set of nodes) are evaluated.

    item
        Specifies the type of values to evaluate:

        U - Displacement

        S - Stress

        EPEL - Elastic strain

    comp
        Specifies the specific component of displacement, stress, or strain
        to evaluate:

        X,Y,Z - Direct components

        XY,YZ,XZ - Shear components (stress and strain only)

        1,2,3 - Principal values (stress and strain only)

        EQV - Equivalent value (stress and strain only)

        SUM - Vector sum (displacement only)

        NORM - L2 norm for the set of nodes (displacement only)

    Notes
    -----
    Up to 50 specifications can be defined for use in a subsequent CYCCALC
    command. If more than 50 specifications are desired, erase the table
    after the CYCCALC operation and add new specifications and repeat the
    CYCCALC command. All the specified nodes, items, and components are
    evaluated for all sectors and the maximum amplitude value output. For
    combined stresses and strains (Comp = 1,2,3 or EQV) or displacement
    vector sum (Comp = SUM), a 360 degree phase sweep is performed at each
    location to determine the maximum.

    Additional POST1 controls are used to refine the specification. For
    component values, components are in the RSYS direction. For shell
    elements, the results are at the SHELL location. For EPEL,EQV, the
    results are based on the EFFNU value on the AVPRIN command. The
    controls active when the CYCCALC command is issued determine the result
    values. If results at another SHELL location are desired, issue the new
    SHELL command and then re-issue the CYCCALC command.

    If a single node is input, the Item/Comp value at that location in each
    sector is output. If a node component is given, then the maximum
    Item/Comp value within the set of nodes of each sector is output, one
    value for each sector (the node of the maximum may vary from sector to
    sector). For stress and strain items, only corner nodes are valid.

    For the displacement norm option (Item = U, Comp = NORM), the L2 norm
    computed from all the nodes in the component is output, one per sector.
    """
    command = f"CYCSPEC,{label},{node},{item},{comp}"
    return self.run(command, **kwargs)


def exoption(self, ldtype="", option="", value="", **kwargs):
    """Specifies the EXPROFILE options for the Mechanical APDL to ANSYS CFX profile file transfer.

    APDL Command: EXOPTION

    Parameters
    ----------
    ldtype
        Load type:

        * ``"SURF"`` : Surface load

        * ``"VOLU"`` : Volume load

    option
        Surface options:

        * ``"Precision"`` : Number of significant digits for the
          fractional part of real data

        * ``"Connectivity"`` : Key to include face connectivity in the
          exported profile file

        * ``"Precision"`` : Number of significant digits after the
          decimal for real data

    value
        Specify the value for either Precision or Connectivity.

        For Precision, specify the number of significant digits. Can
        be any value between 1 to 20, default 8. When 0 or an invalid
        value is specified, the program will use the default value of
        8 and issue a warning message.

        For Connectivity, specify the key to include the element face
        connectivity data for surface loads (does not support volume
        loads):

        * ``"OFF"`` : Do not include the connectivity data in the exported file (default)

        * ``"ON"`` : Include the connectivity data in the exported file

    Notes
    -----
    This command is not supported in Distributed ANSYS.
    """
    return self.run(f"EXOPTION,{ldtype},{option},{value}", **kwargs)


def expand(self, nrepeat="", hindex="", icsys="", sctang="", phase="",
           **kwargs):
    """Displays the results of a modal cyclic symmetry analysis.

    APDL Command: EXPAND

    Parameters
    ----------
    nrepeat
        Number of sector repetitions for expansion. The default is 0 (no
        expansion).

    modal
        Specifies that the expansion is for a modal cyclic symmetry
        analysis.

    hindex
        The harmonic index ID for the results to expand.

    icsys
        The coordinate system number used in the modal cyclic symmetry
        solution. The default is the global cylindrical coordinate system
        (specified via the CSYS command where KCN = 1).

    sctang
        The sector angle in degrees, equal to 360 divided by the number of
        cyclic sectors.

    --
        This field is reserved for future use.

    phase
        The phase angle in degrees to use for the expansion. The default is
        0. Typically, the value is the peak displacement (or stress/strain)
        phase angle obtained via the CYCPHASE command.

    Notes
    -----
    Issue this command to display the results of a modal cyclic symmetry
    analysis.

    When you issue the EXPAND,Nrepeat command, subsequent SET commands read
    data from the results file and expand them to Nrepeat sectors. As long
    as no entities have been modified, this expansion can be negated (that
    is, reverted to single sector) by issuing EXPAND with no arguments. If
    you modify entities and wish to return to the partial model, use the
    Session Editor (see Restoring Database Contents in the Operations
    Guide).

    EXPAND displays the results and allows you to print them, as if for a
    full model. The harmonic index (automatically retrieved from the
    results file) appears in the legend column.

    When plotting or printing element strain energy (SENE), the EXPAND
    command works with brick or tet models only. Element kinetic energy
    (KENE) plotting or printing is not supported.

    EXPAND is a specification command valid only in POST1.  It is
    significantly different from the /CYCEXPAND command in several
    respects, (although you can use either command to display the results
    of a modal cyclic symmetry analysis):

    EXPAND has none of the limitations of the /CYCEXPAND command.

    EXPAND changes the database by modifying the geometry, the nodal
    displacements, and element stresses as they are read from the results
    file, whereas the /CYCEXPAND command does not change the database.

    Caution:: : The EXPAND command creates new nodes and elements;
    therefore, saving (or issuing the /EXIT, ALL command) after issuing the
    EXPAND command can result in large databases.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"EXPAND,{nrepeat},{hindex},{icsys},{sctang},{phase}"
    return self.run(command, **kwargs)


def exprofile(self, ldtype="", load="", value="", pname="", fname="",
              fext="", fdir="", **kwargs):
    """Exports interface loads or loads on selected nodes to an ANSYS CFX

    APDL Command: EXPROFILE
    Profile file.

    Parameters
    ----------
    ldtype
        Load type:

        SURF  - Surface load.

        VOLU  - Volumetric load.

    load
        Surface loads:

        DISP  - Displacement (in a static analysis) or mode shape and global parameters (in a
                modal analysis).

        MODE  - Normalized mode shape and global parameters (in a modal analysis only).

        TEMP  - Temperature.

        HFLU  - Heat flux.

    value
        If a positive integer, specifies the number of the surface or
        volume interface. If 0 (zero), the selected nodes or Named
        Selection are used.

    pname
        Field name in CFX Profile file (32-character maximum). Defaults to
        jobname_bcploadnumber for a surface load and jobname_subdloadnumber
        for volumetric load.

    fname
        The CFX Profile filename (248-character maximum). Defaults to
        jobname_bcploadnumber for a surface load and jobname_subdloadnumber
        for a volumetric load.

    fext
        The Profile file extension (8-character maximum). Defaults to.csv.

    fdir
        The Profile file directory (8-character maximum). Defaults to
        current directory.

    Notes
    -----
    By default, the EXPROFILE command assumes the data it writes to the
    Profile file are in SI units. For models not described in SI units,
    issue the EXUNIT command as needed to write the correct unit labels on
    the Profile file.

    For a modal analysis, if Load = DISP or MODE, global parameters
    including mass, frequency, and maximum displacement are also written to
    the ANSYS CFX Profile file. You should therefore issue the EXUNIT
    command for DISP, TIME, and MASS.

    Verify that the coordinate system is set to the global Cartesian
    (RSYS,0) before using this command.

    To transfer multiple loads across an interface, specify a unique file
    name and extension for each load.

    Force (FORC) and heat generation (HGEN) are per-unit volume.

    For modal analysis, this command does not support the following mode-
    extraction methods (MODOPT): unsymmetric matrix (UNSYM), the damped
    system (DAMP), or the QR-damped system (QRDAMP).

    To write the normalized (instead of non-normalized) mode shapes from a
    modal analysis to the file:

    Use Load = MODE.

    Verify that the mode shapes are normalized to the mass matrix
    (MODOPT,,,,,,OFF), the default behavior.

    Verify that the scale factor is set to 1.0 (SET,,,1.0), the default
    value.

    For loads not specified directly via commands (such as SF and BF),
    loads must first be read into the database (SET or LCASE).

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"EXPROFILE,{ldtype},{load},{value},{pname},{fname},{fext},{fdir}"
    return self.run(command, **kwargs)


def exunit(self, ldtype="", load="", untype="", name="", **kwargs):
    """Specifies the interface load unit labels to be written to the export

    APDL Command: EXUNIT
    file for ANSYS to CFX transfer.

    Parameters
    ----------
    ldtype
        Load type:

        SURF  - Surface load.

        VOLU  - Volumetric load.

    load
        Surface loads:

        DISP  -  Displacement in a static analysis. Mode shape in a modal analysis.

        TIME  - Time. The unit for frequency is the inverse of the unit for time.

        MASS  -  Mass.

        TEMP  - Temperature.

        HFLU  - Heat flux.

    untype
        Unit type:

        COMM  - Predefined unit

        USER  - User-specified unit

    name
        Commonly used predefined unit name or user-specified unit name.

        SI - International System of units (meter-kilogram-second) (default)

        FT -  English System of units (feet-pound-second)

    Notes
    -----
    This command only specifies which unit labels are to be written to the
    file when the EXPROFILE is issued. It does not perform unit
    conversions.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"EXUNIT,{ldtype},{load},{untype},{name}"
    return self.run(command, **kwargs)


def fssparm(self, port1="", port2="", **kwargs):
    """Calculates reflection and transmission properties of a frequency

    APDL Command: FSSPARM
    selective surface.

    Parameters
    ----------
    port1
        Port number of input port. Defaults to 1.

    port2
        Port number of output port. Defaults to 1.

    Notes
    -----
    FSSPARM calculates reflection and transmission coefficients, power
    reflection and transmission coefficients, and return and insertion
    losses of a frequency selective surface.
    """
    command = f"FSSPARM,{port1},{port2}"
    return self.run(command, **kwargs)


def fsum(self, lab="", item="", **kwargs):
    """Sums the nodal force and moment contributions of elements.

    APDL Command: FSUM

    Parameters
    ----------
    lab
        Coordinate system in which to perform summation.

        (blank) - Sum all nodal forces in global Cartesian coordinate system (default).

        RSYS - Sum all nodal forces in the currently active RSYS coordinate system.

    item
        Selected set of nodes.

        (blank) - Sum all nodal forces for all selected nodes (default), excluding contact
                  elements.

        CONT - Sum all nodal forces for contact nodes only.

        BOTH - Sum all nodal forces for all selected nodes, including contact elements.

    Notes
    -----
    Sums and prints, in each component direction for the total selected
    node set, the nodal force and moment contributions of the selected
    elements attached to the node set. Selecting a subset of nodes [NSEL]
    and then issuing this command will give the total force acting on that
    set of nodes (default), excluding surface-to-surface, node-to-surface,
    line-to-line, and line-to-surface contact elements (TARGE169, TARGE170,
    CONTA171, CONTA172, CONTA173, CONTA174, CONTA175, CONTA176, and
    CONTA177).

    Setting ITEM = CONT sums the nodal forces and moment contributions of
    the selected contact elements (CONTA171, CONTA172, CONTA173, CONTA174,
    CONTA175, CONTA176, and CONTA177). Setting ITEM = BOTH sums the nodal
    forces for all selected nodes, including contact elements.

    Nodal forces associated with surface loads are not included. The
    effects of nodal coupling and constraint equations are ignored. Moment
    summations are about the global origin unless another point is
    specified with the SPOINT command. This vector sum is printed in the
    global Cartesian system unless it is transformed [RSYS] and a point is
    specified with the SPOINT command. By default, the sum is done in
    global Cartesian, and the resulting vector is transformed to the
    requested system.

    The LAB = RSYS option transforms each of the nodal forces into the
    active coordinate system before summing and printing. The FORCE command
    can be used to specify which component (static, damping, inertia, or
    total) of the nodal load is to be used. This command output is included
    in the NFORCE command.

    The command should not be used with axisymmetric elements because it
    might calculate a moment where none exists. Consider, for example, the
    axial load on a pipe modeled with an axisymmetric shell element.  The
    reaction force on the end of the pipe is the total force (for the full
    360 degrees) at that location. The net moment about the centerline of
    the pipe would be zero, but the program would incorrectly calculate a
    moment at the end of the element as the force multiplied by the radius.

    The command is not valid for elements that operate solely within the
    nodal coordinate system with 1-D option activated and rotated nodes
    (NROTAT).
    """
    command = f"FSUM,{lab},{item}"
    return self.run(command, **kwargs)


def hfang(self, lab="", phi1="", phi2="", theta1="", theta2="", **kwargs):
    """Defines or displays spatial angles of a spherical radiation surface for

    APDL Command: HFANG
    sound radiation parameter calculations.

    Parameters
    ----------
    lab
        Spatial angle label.

        ANGLE - Define spatial angles (default).

        STATE - Display spatial angles. PHI1, PHI2, THETA1, and THETA2 are ignored.

    phi1, phi2
        Starting and ending ϕ angles (degrees) in the spherical coordinate
        system. Defaults to 0.

    theta1, theta2
        Starting and ending θ angles (degrees) in the spherical coordinate
        system. Defaults to 0.

    Notes
    -----
    Defines or displays spatial angles of a spherical radiation surface.

    Use this command only with PLFAR,Lab = PRES, or PRFAR,Lab = PRES.
    """
    command = f"HFANG,{lab},{phi1},{phi2},{theta1},{theta2}"
    return self.run(command, **kwargs)


def hfsym(self, kcn="", xkey="", ykey="", zkey="", **kwargs):
    """Indicates the presence of symmetry planes for the computation of

    APDL Command: HFSYM
    acoustic fields in the near and far field domains (beyond the finite
    element region).

    Parameters
    ----------
    kcn
        Coordinate system reference number. KCN may be 0 (Cartesian), or
        any previously defined local Cartesian coordinate system number
        (>10). Defaults to 0.

    xkey
        Key for acoustic field boundary condition, as prescribed for the
        solution, corresponding to the x = constant plane:

        None - No sound soft or sound hard boundary conditions (default).

        SSB - Sound soft boundary (pressure = 0).

        SHB - Sound hard boundary (normal velocity = 0).

    ykey
        Key for acoustic field boundary condition, as prescribed for the
        solution, corresponding to the y = constant plane:

        None - No sound soft or sound hard boundary conditions (default).

        SSB - Sound soft boundary (pressure = 0).

        SHB - Sound hard boundary (normal velocity = 0).

    zkey
        Key for acoustic field boundary condition, as prescribed for the
        solution, corresponding to the z = constant plane:

        None - No sound soft or sound hard boundary conditions (default).

        SSB - Sound soft boundary (pressure = 0).

        SHB - Sound hard boundary (normal velocity = 0).

    Notes
    -----
    HFSYM uses the image principle to indicate symmetry planes (x, y, or z
    = constant plane) for acoustic field computations outside the modeled
    domain. A sound hard boundary condition must be indicated even though
    it occurs as a natural boundary condition.

    No menu paths are available for acoustic applications.
    """
    command = f"HFSYM,{kcn},{xkey},{ykey},{zkey}"
    return self.run(command, **kwargs)


def intsrf(self, lab="", **kwargs):
    """Integrates nodal results on an exterior surface.

    APDL Command: INTSRF

    Parameters
    ----------
    lab
        Label indicating degree of freedom to be integrated:

        PRES - Pressure.

        TAUW - Wall shear stress.

        FLOW - Both pressure and wall shear stress.

    Notes
    -----
    Integrates nodal results on a surface.  Use node selection (such as the
    EXT option of the NSEL command) to indicate the surface(s) of element
    faces to be used in the integration.  A surface can be "created" by
    unselecting elements (such as unselecting non-fluid elements that are
    adjacent to fluid elements for the postprocessing of fluid flow
    result).  Element faces attached to the selected nodes will be
    automatically determined.  All nodes on a face must be selected for the
    face to be used.  The integration results will cancel for nodes on
    common faces of adjacent selected elements.

    Integration results are in the active coordinate system (see the RSYS
    command). The type of results coordinate system must match the type
    used in the analysis. However, you may translate and rotate forces and
    moments as needed. Use the *GET command (Utility Menu> Parameters> Get
    Scalar Data) to retrieve the results.
    """
    command = f"INTSRF,{lab}"
    return self.run(command, **kwargs)


def kcalc(self, kplan="", mat="", kcsym="", klocpr="", **kwargs):
    """Calculates stress intensity factors in fracture mechanics analyses.

    APDL Command: KCALC

    Parameters
    ----------
    kplan
        Key to indicate stress state for calculation of stress intensity
        factors:

        0 - Plane strain and axisymmetric condition (default).

        1 - Plane stress condition.

    mat
        Material number used in the extrapolation (defaults to 1).

    kcsym
        Symmetry key:

        0 or 1  - Half-crack model with symmetry boundary conditions [DSYM] in the crack-tip
                  coordinate system.  KII = KIII = 0.  Three nodes are
                  required on the path.

        2 - Like 1 except with antisymmetric boundary conditions (KI = 0).

        3 - Full-crack model (both faces).  Five nodes are required on the path (one at the
            tip and two on each face).

    klocpr
        Local displacements print key:

        0 - Do not print local crack-tip displacements.

        1 - Print local displacements used in the extrapolation technique.

    Notes
    -----
    Calculates the stress intensity factors (KI, KII, and KIII) associated
    with homogeneous isotropic linear elastic fracture mechanics.  A
    displacement extrapolation method is used in the calculation (see POST1
    -  Crack Analysis in the Mechanical APDL Theory Reference). This method
    assumes that the displacement calculations are for the plane strain
    state. If the displacement calculations are performed using a plane
    stress formulation, the calculation of the stress intensity factors can
    be converted to the plane strain state by using KPLAN = 1. ANSYS Uses
    minor Poisson's ratio (MP,NUXY) for the stress intensity factor
    calculation, therefore the material's Poisson's ratio must be defined
    using MP,NUXY command. The PATH and PPATH commands must be used to
    define a path with the crack face nodes (NODE1 at the crack tip, NODE2
    and NODE3 on one face, NODE4 and NODE5 on the other (optional) face).
    A crack-tip coordinate system, having x parallel to the crack face (and
    perpendicular to the crack front) and y perpendicular to the crack
    face, must be the active RSYS and CSYS before KCALC is issued.
    """
    command = f"KCALC,{kplan},{mat},{kcsym},{klocpr}"
    return self.run(command, **kwargs)


def nforce(self, item="", **kwargs):
    """Sums the nodal forces and moments of elements attached to nodes.

    APDL Command: NFORCE

    Parameters
    ----------
    item
        Specifies the selected set of nodes for summing forces and moments
        for contact elements.

        (blank) - Sums the nodal forces of elements for all selected nodes and excludes contact
                  elements (elements 169-177).

        CONT - Sums the nodal forces of elements for contact nodes only.

        BOTH - Sums the nodal forces of elements for all selected nodes, including contact
               elements.

    Notes
    -----
    Sums and prints, in each component direction for each selected node,
    the nodal force and moment contributions of the selected elements
    attached to the node.  If all elements are selected, the sums are
    usually zero except where constraints or loads are applied.  The nodal
    forces and moments may be displayed [/PBC,FORC and /PBC,MOME].  Use
    PRESOL to print nodal forces and moments on an element-by-element
    basis.  You can use the FORCE command to specify which component
    (static, damping, inertia, or total) of the nodal load is to be used.
    Nodal forces associated with surface loads are not included.

    This vector sum is printed in the global Cartesian system. Moment
    summations are about the global origin unless another point is
    specified with the SPOINT command.  The summations for each node are
    printed in the global Cartesian system unless transformed [RSYS].  This
    command is generally not applicable to axisymmetric models because
    moment information from the NFORCE command is not correct for
    axisymmetric elements.

    Selecting a subset of elements [ESEL] and then issuing this command
    will give the forces and moments required to maintain equilibrium of
    that set of elements.  The effects of nodal coupling and constraint
    equations are ignored. The option ITEM = CONT provides the forces and
    moments for the contact elements (CONTA171, CONTA172, CONTA173,
    CONTA174, CONTA175, CONTA176, and CONTA177). Setting ITEM = BOTH
    provides the forces and moments for all selected nodes, including
    contact elements.

    This command also includes the FSUM command function which vectorially
    sums and prints, in each component direction for the total selected
    node set, the nodal force and moment contributions of the selected
    elements attached to the selected node set.
    """
    command = f"NFORCE,{item}"
    return self.run(command, **kwargs)


def nldpost(self, label="", key="", fileid="", prefix="", **kwargs):
    """Gets element component information from nonlinear diagnostic files.

    APDL Command: NLDPOST

    Parameters
    ----------
    label
        Specifies the type of command operation:

        EFLG - Element flag for nonlinear diagnostics.

        NRRE - Newton-Raphson residuals.

    key
        Specifies the command action:

        STAT - List information about the diagnostic files (Jobname.ndxxx or Jobname.nrxxx) in
               the current directory.

        For Label = EFLG, the listing gives a summary that associates the loadstep, substep, time, equilibrium iteration number, cumulative iteration number, and the number of elements that fail each criteria with a specific file ID (Jobname.ndxxx). Use the list to create element components (via the CM option) based on the cumulative iteration number. - For Label = NRRE, the listing provides a summary that associates the loadstep,
                          substep, time, equilibrium iteration number, and
                          cumulative iteration number with a specific file
                          ID (Jobname.nrxxx).  Use the list to identify the
                          respective file ID for creating Newton-Raphson
                          residual contour plots (PLNSOL,NRRE,…,FileID).

        DEL - Delete Jobname.ndxxx or Jobname.nrxxx files in the working directory, if any
              exist.

        CM - Create components for elements that violate criteria. This value is valid only
             when Label = EFLG.

    fileid
        Valid only when Label = EFLG and Key = CM, this value specifies
        file IDs:

        IDnum - The file ID number. Creates the element components from the diagnostic files
                corresponding to the specified file ID number in the
                working directory.

        ALL - Creates element components from all available diagnostic files residing in the
              working directory. This value is the default if you do not
              specify an IDnum value.

    prefix
        Sets the prefix name for components. Specify up to 21 alphanumeric
        characters.

    Notes
    -----
    Based on the nonlinear diagnostic results (created via the NLDIAG,EFLG
    command), the NLDPOST command creates element components with
    predefined names.

    The following table lists the diagnostic criteria and component names
    (with specified prefix and without). Here xxx corresponds to the file
    ID (FileID) of Jobname.ndxxx or Jobname.nrxxx.

    If you have trouble viewing specific element components, see Viewing
    Hidden Element Components in the Basic Analysis Guide.

    For more information, see Performing Nonlinear Diagnostics.
    """
    command = f"NLDPOST,{label},{key},{fileid},{prefix}"
    return self.run(command, **kwargs)


def plcamp(self, option="", slope="", unit="", freqb="", cname="",
           stabval="", keyallfreq="", keynegfreq="", **kwargs):
    """Plots Campbell diagram data for applications involving rotating

    APDL Command: PLCAMP
    structure dynamics.

    Parameters
    ----------
    option
        Flag to activate or deactivate sorting of forward or backward whirl
        frequencies:

        0 (OFF or NO) - No sorting.

        1 (ON or YES) - Sort. This value is the default.

    slope
        The slope of the line to be printed. This value must be positive.

        SLOPE > 0  - The line represents the number of excitations per revolution of the rotor. For
                     example, SLOPE = 1 represents one excitation per
                     revolution, usually resulting from unbalance.

        SLOPE = 0  - The line represents the stability threshold for stability values or logarithmic
                     decrements printout (STABVAL = 1 or 2)

    unit
        Specifies the unit of measurement for rotational angular
        velocities:

        RDS - Rotational angular velocities in radians per second (rad/s). This value is the
              default.

        RPM - Rotational angular velocities in revolutions per minute (RPMs).

    freqb
        The beginning, or lower end, of the frequency range of interest.
        The default is zero.

    cname
        The rotating component name.

    stabval
        Flag to plot the stability values:

        0 (OFF or NO) - Plot the frequencies (the imaginary parts of the eigenvalues in Hz). This value
                        is the default.

        1 (ON or YES) - Plot the stability values (the real parts of the eigenvalues in Hz).

        2 - Plot the logarithmic decrements.

    keyallfreq
        Key to specify if all frequencies above FREQB are plotted:

        0 (OFF or NO) - A maximum of 10 frequencies are plotted. This value is the default.

        1 (ON or YES) - All frequencies are plotted.

    keynegfreq
        Key to specify if the negative frequencies are plotted. It only
        applies to solutions obtained with the damped eigensolver (Method =
        DAMP on the MODOPT command):

        0 (OFF or NO) - Only positive frequencies are plotted. This value is the default.

        1 (ON or YES) - Negative and positive frequencies are plotted.

    Notes
    -----
    The following items are required when generating a Campbell diagram:

    Take the gyroscopic effect into account by issuing the CORIOLIS command
    in the SOLUTION module.

    Run a modal analysis using the QR damped (MODOPT,QRDAMP) or damped
    (MODOPT,DAMP) method. Complex eigenmodes are necessary
    (MODOPT,QRDAMP,,,,Cpxmod  = ON), and you must specify the number of
    modes to expand (MXPAND).

    Define two or more load step results with an ascending order of
    rotational velocity (OMEGA or CMOMEGA).

    In some cases where modes are not in the same order from one load step
    to the other, sorting the frequencies (Option = 1) can help to obtain a
    correct plot. Sorting is based on the comparison between complex mode
    shapes calculated at two successive load steps.

    At each load step, the application compares the mode shape to the loads
    at other load steps to determine whirl direction at the load step. If
    applicable, a label appears (in the plot legend) representing each
    whirl mode (BW for backward whirl and FW for forward whirl).

    At each load step, the program checks for instability (based on the
    sign of the real part of the eigenvalue). The labels "stable" or
    "unstable" appear in the plot legend for each frequency curve.

    The rotational velocities of a named component (Cname) are displayed on
    the X-axis.

    For information on plotting a Campbell diagram for a prestressed
    structure, see Solving for a Subsequent Campbell Analysis of a
    Prestressed Structure Using the Linear Perturbation Procedure in the
    Rotordynamic Analysis Guide.

    In general, plotting a Campbell diagram is recommended only when your
    analysis is performed in a stationary reference frame
    (CORIOLIS,,,,RefFrame = ON).

    For a usage example of the PLCAMP command, see Campbell Diagram in the
    Rotordynamic Analysis Guide.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"PLCAMP,{option},{slope},{unit},{freqb},{cname},{stabval},{keyallfreq},{keynegfreq}"
    return self.run(command, **kwargs)


def plcfreq(self, spec="", sectbeg="", sectend="", **kwargs):
    """Plots the frequency response for the given CYCSPEC specification.

    APDL Command: PLCFREQ

    Parameters
    ----------
    spec
        CYCSPEC specification number (ordered 1 to N in the order input;
        use CYCSPEC,LIST to view the current list order). Defaults to 1.

    sectbeg
        Beginning sector number to plot. Defaults to 1.

    sectend
        Ending sector number to plot. Defaults to the total number of
        sectors expanded (/CYCEXPAND).

    Notes
    -----
    Following a cyclic mode-superposition harmonic analysis, this command
    plots the result item given by a CYCSPEC specification versus the
    harmonic frequency, one curve for each of the specified sectors. A
    CYCCALC command must have been issued prior to this command.
    """
    command = f"PLCFREQ,{spec},{sectbeg},{sectend}"
    return self.run(command, **kwargs)


def plchist(self, spec="", freqpt="", **kwargs):
    """Plots a histogram of the frequency response of each sector for the

    APDL Command: PLCHIST
    given CYCSPEC specification.

    Parameters
    ----------
    spec
        CYCSPEC specification number (ordered 1 to N in the order input;
        use CYCSPEC,LIST to view the current list order). Defaults to 1.

    freqpt
        Harmonic frequency point to plot (the data set number NSET or
        CUMULATIVE on SET,LIST). Defaults to the current SET frequency.

    Notes
    -----
    Following a cyclic mode-superposition harmonic analysis, this command
    creates a histogram plot of the result item given by a CYCSPEC
    specification versus the sector number. A CYCCALC command must have
    been issued prior to this command.
    """
    command = f"PLCHIST,{spec},{freqpt}"
    return self.run(command, **kwargs)


def plfar(self, lab="", option="", phi1="", phi2="", nph1="", theta1="",
          theta2="", ntheta="", val1="", val2="", val3="", **kwargs):
    """Plots pressure far fields and far field parameters.

    APDL Command: PLFAR

    Parameters
    ----------
    lab
        Parameters to plot :

        PRES - Acoustic parameters

        PROT - Acoustic parameters with the y-axis rotated extrusion

    option
        Plot option, based on the specified plot parameter type:

    phi1, phi2
        Starting and ending φ angles (degrees) in the spherical coordinate
        system. Defaults to 0.

    nphi
        Number of divisions between the starting and ending φ angles for
        data computations. Defaults to 0.

    theta1, theta2
        Starting and ending θ angles (degrees) in the spherical coordinate
        system. Defaults to 0 in 3-D and 90 in 2-D.

    ntheta
        Number of divisions between the starting and ending θ angles for
        data computations. Defaults to 0.

    val1
        Radius of the sphere surface. Used only when Option = SUMC, SUMP,
        PHSC, PHSP, SPLC, SPLP, SPAC, SPAP, PSCT, PSPL, TSCT, or TSPL.

    val2
        When Option = SPLC, SPLP, SPAC, or SPAP: Reference rms sound
        pressure. Defaults to 2x10-5 Pa.

    val3
        When Lab = PRES: Thickness of 2-D model extrusion in the z
        direction (no default).

    Notes
    -----
    The PLFAR command plots pressure far fields and far field parameters as
    determined by the equivalent source principle. Use this command to plot
    pressure and acoustic    parameters. See the HFSYM command for the
    model symmetry and the     HFANG command for spatial radiation angles.

    When Option = PWL no plot is generated, but the sound power level
    appears in the output.

    To retrieve saved equivalent source data, issue the
    SET,Lstep,Sbstep,,REAL command.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"PLFAR,{lab},{option},{phi1},{phi2},{nph1},{theta1},{theta2},{ntheta},{val1},{val2},{val3}"
    return self.run(command, **kwargs)


def plmc(self, lstep="", sbstep="", timfrq="", kimg="", hibeg="", hiend="",
         **kwargs):
    """Plots the modal coordinates from a mode-superposition solution.

    APDL Command: PLMC

    Parameters
    ----------
    lstep, sbstep
        Plot the solution identified as load step LSTEP and substep SBSTEP

    timfrq
        As an alternative to LSTEP and SBSTEP, plot the solution at the
        time value TIMFRQ (for ANTYPE,TRANS) or frequency value TIMFRQ (for
        ANTYPE,HARMIC). LSTEP and SBSTEP should be left blank.

    kimg
        If 0 (or blank), plot the real solution. If 1, plot the imaginary
        solution. Only valid for ANTYPE,HARMIC.

    hibeg, hiend
        For cyclic symmetry solutions, plot the solutions in the harmonic
        index solution range HIbeg to HIend. Defaults to all harmonic
        indices (all modes).

    Notes
    -----
    PLMC plots a histogram of the modal coordinates (the response
    amplitudes applied to each mode shape) at a certain time point
    (transient analyses) or frequency point (harmonic analyses). The
    absolute values of the modal coordinates are plotted. Use /XRANGE to
    plot only modes in a certain range, if desired.

    For transient analyses, the Jobname.RDSP file must be available. For
    harmonic analyses, the Jobname.RFRQ must be available. No SET command
    is required and no expansion pass is required.

    For a cyclic harmonic mode-superposition analysis, use the CYCFILES
    command to identify the Jobname.RFRQ and modal Jobname.RST file. You
    may limit the plot to display only those modes in a certain harmonic
    index range. The modes having the same harmonic index are each plotted
    in a unique color. If there are less than 10 harmonic indices, they are
    identified in the graphics legend.

    This is a graphical representation of the optional Jobname.MCF text
    file. (see the TRNOPT and HROPT commands). For more information on
    modal coordinates, see Mode-Superposition Method in the Mechanical APDL
    Theory Reference.
    """
    command = f"PLMC,{lstep},{sbstep},{timfrq},{kimg},{hibeg},{hiend}"
    return self.run(command, **kwargs)


def plnear(self, lab="", opt="", kcn="", val1="", val2="", val3="",
           val4="", val5="", val6="", val7="", val8="", val9="", **kwargs):
    """Plots the pressure in the near zone exterior to the equivalent source

    APDL Command: PLNEAR
    surface.

    Parameters
    ----------
    lab
        Plot the maximum pressure or sound pressure level:

        SPHERE - on the spherical structure

        PATH - along the path

    opt
        PSUM

        PSUM  - Maximum complex pressure for acoustics.

        PHAS  - Phase angle of complex pressure for acoustics.

        SPL  - Sound pressure level for acoustics.

        SPLA - A-weighted sound pressure level for acoustics (dBA).

    kcn
        KCN is the coordinate system reference number. It may be 0
        (Cartesian) or any previously defined local coordinate system
        number (>10). Defaults to 0.

    val1, val2, val3, . . . , val9
        For LAB = SPHERE:

        VAL1 - Radius of spherical surface in spherical coordinate system.

        VAL2 - Starting φ angle (degree) in the spherical coordinate system. Defaults to 0.

        VAL3 - Ending φ angle (degree) in the spherical coordinate system. Defaults to 0.

        VAL4 - Number of divisions between the starting and ending φ angles for data
               computations. Defaults to 0.

        VAL5 - Starting θ angle (degrees) in the spherical coordinate system. Defaults to 0 in
               3-D and 90 in 2-D extension.

        VAL6 - Ending θ angle (degrees) in the spherical coordinate system. Defaults to 0 in
               3-D and 90 in 2-D extension.

        VAL7 - Number of divisions between the starting and ending θ angles for data
               computations. Defaults to 0.

        VAL8 - Reference rms sound pressure. Defaults to 2x10-5 Pa.

        VAL9 - Thickness of 2-D model extension in z direction (defaults to 0).

    Notes
    -----
    PLNEAR uses the equivalent source principle to calculate the pressure
    in the near zone exterior to the equivalent source surface (flagged
    with the Maxwell surface flag in the preprocessor) for one of the
    following locations:

    A spherical surface in the KCN coordinate system

    A path defined by the PATH and PPATH commands

    To plot the pressure results for a path, use the PLPAGM or PLPATH
    commands. See the HFSYM command for the model symmetry.

    To retrieve saved equivalent source data, issue the
    SET,Lstep,Sbstep,,REAL command.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"PLNEAR,{lab},{opt},{kcn},{val1},{val2},{val3},{val4},{val5},{val6},{val7},{val8},{val9}"
    return self.run(command, **kwargs)


def plzz(self, rotvel="", deltarotvel="", **kwargs):
    """Plots the interference diagram from a cyclic modal analysis.

    APDL Command: PLZZ

    Parameters
    ----------
    rotvel
        Rotational speed in revolutions per minute (RPM) used to define the
        speed line. If blank, use the rotational speed (from OMEGA)
        specified in the prestressing step of the linear perturbation
        analysis. If explicitly input as 0, or if the linear perturbation
        was not used, no speed lines are plotted.

    deltarotvel
        Adds speed lines about the RotVel speed line corresponding to
        RotVel ± DeltaRotVel. Only plotted if RotVel is known.

    Notes
    -----
    PLZZ plots the cyclic modal frequencies as points on a frequency vs.
    harmonic index (nodal diameter) graph. If rotational speed (RotVel) is
    provided, the speed line is also plotted, leading to the interference
    diagram (also known as the SAFE or ZZENF diagram). If DeltaRotVel is
    also provided, two additional speed lines are plotted, enveloping the
    safe speed line itself.

    For more information, see Postprocessing a Modal Cyclic Symmetry
    Analysis in the Cyclic Symmetry Analysis Guide.
    """
    command = f"PLZZ,{rotvel},{deltarotvel}"
    return self.run(command, **kwargs)


def pras(self, quantity="", loadstep="", substep="", **kwargs):
    """Calculates a specified acoustic quantity on the selected exterior

    APDL Command: PRAS
    surface or the frequency-band sound pressure level (SPL).

    Parameters
    ----------
    quantity
        The acoustic quantity to calculate:

        SIMP - Specific acoustic impedance on the selected surface.

        AIMP  - Acoustic impedance on the selected surface.

        MIMP  - Mechanical impedance on the selected surface.

        PRES  - Average pressure on the selected surface.

        FORC  - Force on the selected surface.

        POWE  - Acoustic power on the selected surface.

        BSPL  - Frequency-band sound pressure level.

        BSPA  - A-weighted frequency-band sound pressure level.

    loadstep
        Specified load step. Default = 1.

    substep
        Specified substep. Default = All substeps at the specified load
        step. Not valid for Quantity = BSPL or BSPA.

    Notes
    -----
    The PRAS command calculates a specified acoustic quantity on the
    selected exterior surface in postprocessing. The calculation is based
    on the pressure and velocity solution or the frequency-band sound
    pressure level (SPL).

    The total pressure and velocity are used if the selected surface is the
    excitation source surface. To calculate the incoming and outgoing
    acoustic power, and other sound power parameters, on the excitation
    source surface, issue the SF,,PORT and SPOWER commands.

    The sound pressure level of the octave bands and general frequency band
    (defined via the HARFRQ command) is calculated at the selected nodes in
    the model.
    """
    command = f"PRAS,{quantity},{loadstep},{substep}"
    return self.run(command, **kwargs)


def prcamp(self, option="", slope="", unit="", freqb="", cname="",
           stabval="", keyallfreq="", keynegfreq="", **kwargs):
    """Prints Campbell diagram data for applications involving rotating

    APDL Command: PRCAMP
    structure dynamics.

    Parameters
    ----------
    option
        Flag to activate or deactivate sorting of forward or backward whirl
        frequencies:

        0 (OFF or NO) - No sorting.

        1 (ON or YES) - Sort. This value is the default.

    slope
        The slope of the line to be printed. This value must be positive.

        SLOPE > 0  - The line represents the number of excitations per revolution of the rotor. For
                     example, SLOPE = 1 represents one excitation per
                     revolution, usually resulting from unbalance.

        SLOPE = 0  - The line represents the stability threshold for stability values or logarithmic
                     decrements printout (STABVAL = 1 or 2)

    unit
        Specifies the unit of measurement for rotational angular
        velocities:

        RDS - Rotational angular velocities in radians per second (rad/s). This value is the
              default.

        RPM - Rotational angular velocities in revolutions per minute (RPMs).

    freqb
        The beginning, or lower end, of the frequency range of interest.
        The default is zero.

    cname
        The rotating component name.

    stabval
        Flag to print the stability values:

        0 (OFF or NO) - Print the frequencies (the imaginary parts of the eigenvalues in Hz). This
                        value is the default.

        1 (ON or YES) - Print the stability values (the real parts of the eigenvalues in Hz).

        2 - Print the logarithmic decrements.

    keyallfreq
        Key to specify if all frequencies above FREQB are printed out:

        0 (OFF or NO) - A maximum of 10 frequencies are printed out. They correspond to the frequencies
                        displayed via the PLCAMP command. This value is the
                        default.

        1 (ON or YES) - All frequencies are printed out.

    keynegfreq
        Key to specify if the negative frequencies are printed out. It only
        applies to solutions obtained with the damped eigensolver
        (Method=DAMP on the MODOPT command):

        0 (OFF or NO) - Only positive frequencies are printed out. This value is the default.

        1 (ON or YES) - Negative and positive frequencies are printed out.

    Notes
    -----
    The following items are required when generating a Campbell diagram:

    Take the gyroscopic effect into account by issuing the CORIOLIS command
    in the SOLUTION module.

    Run a modal analysis using the QR damped (MODOPT,QRDAMP) or damped
    (MODOPT,DAMP) method. Complex eigenmodes are necessary
    (MODOPT,QRDAMP,,,,Cpxmod  = ON), and you must specify the number of
    modes to expand (MXPAND).

    Define two or more load step results with an ascending order of
    rotational velocity (OMEGA or CMOMEGA).

    In some cases where modes are not in the same order from one load step
    to the other, sorting the frequencies (Option = 1) can help to obtain a
    correct printout. Sorting is based on the comparison between complex
    mode shapes calculated at two successive load steps.

    At each load step, the application compares the mode shape to the loads
    to determine the whirl direction. If applicable, a label appears (on
    the rows of output data) representing the whirl mode (BW for backward
    whirl and FW for forward whirl).

    If you specify a non-zero slope (SLOPE > 0), the command prints the
    critical speeds corresponding to the intersection points of the
    frequency curves and the added line. In the case of a named component
    (Cname), critical speeds relate to the rotational velocity of the
    component. Critical speeds are available only if the frequencies are
    printed (STABVAL = OFF).

    If you specify a zero slope (SLOPE = 0), the command prints the
    stability threshold corresponding to the sign change of the stability
    values (or logarithmic decrements). In the case of a named component
    (Cname), stability thresholds relate to the rotational velocity of the
    component. Stability thresholds are available only if the stability
    values or logarithmic decrements are printed (STABVAL = 1 or 2).

    At each load step, the program checks for instability (based on the
    sign of the real part of the eigenvalue). The label "U" appears on the
    printout for each unstable frequency.

    If specified, the rotational velocities of the named component (Cname)
    are printed out along with the natural frequencies.

    In general, printing a Campbell diagram is recommended only when your
    analysis is performed in a stationary reference frame
    (CORIOLIS,,,,RefFrame = ON).

    For information on printing a Campbell diagram for a prestressed
    structure, see Solving for a Subsequent Campbell Analysis of a
    Prestressed Structure Using the Linear Perturbation Procedure in the
    Rotordynamic Analysis Guide.

    For a usage example of the companion command PLCAMP (used for plotting
    a Campbell diagram), see Example Campbell Diagram Analysis.

    For more information on Campbell diagram generation, see Campbell
    Diagram in the Rotordynamic Analysis Guide.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"PRCAMP,{option},{slope},{unit},{freqb},{cname},{stabval},{keyallfreq},{keynegfreq}"
    return self.run(command, **kwargs)


def prfar(self, lab="", option="", phi1="", phi2="", nph1="", theta1="",
          theta2="", ntheta="", val1="", val2="", val3="", **kwargs):
    """Prints pressure far fields and far field parameters.

    APDL Command: PRFAR

    Parameters
    ----------
    lab
        Parameters to print:

        PRES - Acoustic parameters

        PROT - Acoustic parameters with the y-axis rotated extrusion

    option
        Print option, based on the specified print parameter type:

    phi1, phi2
        Starting and ending φ angles (degrees) in the spherical coordinate
        system. Defaults to 0.

    nphi
        Number of divisions between the starting and ending φ angles for
        data computations. Defaults to 0.

    theta1, theta2
        Starting and ending θ angles (degrees) in the spherical coordinate
        system. Defaults to 0 in 3-D and 90 in 2-D.

    ntheta
        Number of divisions between the starting and ending θ angles for
        data computations. Defaults to 0.

    val1
        Radius of the sphere surface. Used only when Option = SUMC, PHSC,
        SPLC, SPAC, PSCT, or TSCT.

    val2
        When Option = SPLC or SPAC: Reference rms sound pressure. Defaults
        to 2x10-5 Pa.

    val3
        When Lab = PRES: Thickness of 2-D model extrusion in the z
        direction (no default).

    Notes
    -----
    The PRFAR command prints pressure far fields and far field parameters
    as    determined by the equivalent source principle. Use this command
    to print pressure and acoustic    parameters. See the HFSYM command for
    the model symmetry and the     HFANG command for spatial radiation
    angles.

    To retrieve saved equivalent source data, issue the
    SET,Lstep,Sbstep,,REAL command.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"PRFAR,{lab},{option},{phi1},{phi2},{nph1},{theta1},{theta2},{ntheta},{val1},{val2},{val3}"
    return self.run(command, **kwargs)


def prnear(self, lab="", opt="", kcn="", val1="", val2="", val3="",
           val4="", val5="", val6="", val7="", val8="", val9="", **kwargs):
    """Prints the pressure in the near zone exterior to the equivalent source

    APDL Command: PRNEAR
    surface.

    Parameters
    ----------
    lab
        Print the maximum pressure or sound pressure level:

        POINT - at the point (x,y,z)

        SPHERE - on the spherical structure

        PATH - along the path

    opt
        PSUM

        PSUM  - Maximum complex pressure for acoustics.

        PHAS  - Phase angle of complex pressure for acoustics.

        SPL  - Sound pressure level for acoustics.

        SPLA - A-weighted sound pressure level for acoustics (dBA).

    kcn
        KCN is the coordinate system reference number. It may be 0
        (Cartesian) or any previously defined local coordinate system
        number (>10). Defaults to 0.

    val1, val2, val3, . . . , val9
        For Lab = POINT:

        VAL1 - x coordinate value

        VAL2 - y coordinate value

        VAL3 - z coordinate value

        VAL4 - VAL8 - not used

        VAL9 - Thickness of model in z direction (defaults to 0).

    Notes
    -----
    The command uses the equivalent source principle to calculate the
    pressure in the near zone exterior to the equivalent source surface
    (flagged with the Maxwell surface flag in the preprocessor) for one of
    the following locations:

    A point X, Y, Z in the KCN coordinate system

    A spherical surface in the KCN coordinate system

    A path defined by the PATH and PPATH commands

    To list the pressure results for a path, use the PRPATH command. See
    HFSYM command for the model symmetry.

    To retrieve saved equivalent source data, issue the
    SET,Lstep,Sbstep,,REAL command.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"PRNEAR,{lab},{opt},{kcn},{val1},{val2},{val3},{val4},{val5},{val6},{val7},{val8},{val9}"
    return self.run(command, **kwargs)


def reswrite(self, fname="", cflag="", **kwargs):
    """Appends results data from the database to a results file.

    APDL Command: RESWRITE

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum,
        including the characters needed for the directory
        path). An unspecified directory path defaults to the
        working directory; in this case, you can use all 248
        characters for the file name.  The file name extension
        varies as follows:

        - .RST for structural, fluid, or coupled-field analyses
         - .RTH for thermal or electrical analyses
         - .RMG for magnetic analyses

    cflag
        0 - The complex results flag is set to 0 in the results
        file header. This is the default option.

        1 - The complex results flag is set to 1 in the results
        file header.

    Notes
    -----
    The RESWRITE command appends a data set to the specified file
    by writing the results data currently in the database. If the
    intended results file does not exist, it will be created and
    will include the geometry records. The current load step,
    substep, and time (or frequency) value are maintained. All
    data (summable and nonsummable) are written.

    When complex results are appended, cFlag must be set to 1 so
    that the header is consistent with the results written on the
    file.

    The command is primarily intended for use in a top-down
    substructuring analysis, where the full model is resumed and
    the results data read from the use pass results file (SET),
    and subsequently from all substructure expansion pass results
    files (APPEND). The full set of data in memory can then be
    written out via the RESWRITE command to create a complete
    results file (as though you had run a nonsubstructured
    analysis).

    The RESWRITE command can also be used to write a global
    results file for a distributed parallel (Distributed ANSYS)
    solution. This should only be necessary if the RESCOMBINE
    command was used to combine results from local results files
    into the database. The RESWRITE command can then be used to
    write the combined results into a new results file.  This new
    results file will essentially contain the current set of
    results data for the entire (i.e., global) model.
    """

    return self.run(f"RESWRITE,{fname},,,,{cflag}", **kwargs)


def rmflvec(self, **kwargs):
    """Writes eigenvectors of fluid nodes to a file for use in damping

    APDL Command: RMFLVEC
    parameter extraction.

    Notes
    -----
    RMFLVEC extracts the modal information from the modal results file for
    all nodes specified in a node component called 'FLUN'. This component
    should include all nodes which are located at the fluid-structural
    interface. Mode shapes, element normal orientation, and a scaling
    factor are computed and stored in a file Jobname.EFL. For damping
    parameter extraction, use the DMPEXT command macro. See Introduction
    for more information on thin film analyses.

    FLUID136 and FLUID138 are used to model the fluid interface. Both the
    structural and fluid element types must be active. The fluid interface
    nodes must be grouped into a component 'FLUN'. A results file of the
    last modal analysis must be available.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"RMFLVEC,"
    return self.run(command, **kwargs)


def rsplit(self, option="", label="", name1="", name2="", name3="",
           name4="", name5="", name6="", name7="", name8="", name9="",
           name10="", name11="", name12="", name13="", name14="",
           name15="", name16="", **kwargs):
    """Creates one or more results file(s) from the current results file based

    APDL Command: RSPLIT
    on subsets of elements.

    Parameters
    ----------
    option
        Specify which results to include for the subset of elements.

        ALL  - Write all nodal and element results based on the subset of elements (default).

        EXT  - Write only the nodal and element results that are on the exterior surface of
               the element subset. The results data will be averaged as in
               PowerGraphics (see AVRES) when this results file is brought
               into POST1. Only valid for solid elements.

    label
        Define where the element subset is coming from.

        ALL  - Use all selected element components (CMSEL) (default).

        ESEL  - Use the currently selected (ESEL) set of elements. Name1 defines the results
                file name.

        LIST - Use Name1 to Name16 to list the element component and/or assembly names (that
               contain element components).

    name1, name2, name3, . . . , name16
        Up to 16 element component and/or assembly names (that contain
        element components).

    Notes
    -----
    Results files will be named based on the element component or assembly
    name, e.g., Cname.rst, except for the ESEL option, for which you must
    specify the results file name (no extension) using the Name1 field.
    Note that the .rst filename will be written in all uppercase letters
    (CNAME.rst) (unless using the ESEL option); when you read the file, you
    must specify the filename using all uppercase letters (i.e.,
    file,CNAME). You may repeat the RSPLIT command as often as needed. All
    results sets on the results file are processed. Use /AUX3 to produce a
    results file with just a subset of the results sets.

    Use INRES to limit the results data written to the results files.

    The subset geometry is also written so that no database file is
    required to postprocess the subset results files. You must not resume
    any database when postprocessing one of these results files. The input
    results file must have geometry written to it (i.e., do not use
    /CONFIG,NORSTGM,1).

    Applied forces and reaction forces are not apportioned if their nodes
    are shared by multiple element subsets. Their full values are written
    to each results file.

    Each results file renumbers its nodes and elements starting with 1.

    This feature is useful when working with large models. For more
    information on the advantages and uses of the RSPLIT command, see
    Splitting Large Results Files in the Basic Analysis Guide.
    """
    command = f"RSPLIT,{option},{label},{name1},{name2},{name3},{name4},{name5},{name6},{name7},{name8},{name9},{name10},{name11},{name12},{name13},{name14},{name15},{name16}"
    return self.run(command, **kwargs)


def rstmac(self, file1="", lstep1="", sbstep1="", file2="", lstep2="",
           sbstep2="", maclim="", cname="", keyprint="", **kwargs):
    """APDL Command: RSTMAC

    Calculates modal assurance criterion (MAC) and matches nodal solutions
    from two results files or from one results file and one universal
    format file.

    Parameters
    ----------
    file1
        File name (32 characters maximum) corresponding to the first
        results file (.rst or .rstp file). If the file name does not
        contain the extension, it defaults to .rst.

    lstep1
        Load step number of the results to be read in File1.

        N  - Reads load step N. Defaults to 1.

    sbstep1
        Substep number of the results to be read in File1.

        N  - Reads substep N.

        All  - Reads all substeps. This value is the default.

    file2
        File name (32 characters maximum) corresponding to the second file
        (.rst, .rstp, or .unv file). If the file name does not contain the
        extension, it defaults to .rst.

    lstep2
        Load step number of the results to be read in File2.

        N  - Reads load step N. Defaults to 1.

    sbstep2
        Substep number of the results to be read in File2.

        N  - Reads substep N.

        All  - Reads all substeps. This value is the default.

    maclim
        Smallest acceptable MAC value. Must be  0 and  1. The default value
        is 0.90.

    cname
        Name of the component from the first file (File1). The component
        must be based on nodes. If unspecified, all nodes are matched and
        used for MAC calculations. If a component name is specified, only
        nodes included in the specified component are used. Not applicable
        to node mapping (TolerN=-1).

    keyprint
        Printout options:

        0  - Printout matched solutions table. This value is the default.

        1  - Printout matched solutions table and full MAC table.

        2  - Printout matched solutions table, full MAC table and matched nodes table.

    Notes
    -----
    The RSTMAC command allows the comparison of the solutions from
    either:

    Two different results files

    One result file and one universal format file

    The modal assurance criterion (MAC) is used.

    The meshes read on File1 and File2 may be different. If TolerN>0,
    the nodes are matched. This is the default. If TolerN = -1, the
    nodes are mapped and the solutions are interpolated from File1.

    Units and coordinate systems must be the same for both
    models. When a universal format file is used, the nodal
    coordinates can be scaled using UNVscale.

    The corresponding database file (.db) for File1 must be resumed
    before running the command only if a component (Cname) is used or
    if the nodes are mapped (TolerN = -1).

    Results may be real or complex; however, if results from File1
    have a different type from results in File2, only the real parts
    of the solutions are taken into account in MAC calculations. The
    analysis type can be arbitrary.

    Only structural degrees of freedom are considered. Degrees of
    freedom can vary between File1 and File2, but at least one common
    degree of freedom must exist.

    When node mapping and solution interpolation is performed
    (TolerN=-1), File1 must correspond to a model meshed in solid
    and/or shell elements.  Other types of elements can be present but
    the node mapping is not performed for those
    elements. Interpolation is performed on UX, UY, and UZ degrees of
    freedom.

    The solutions read on the results files are not all written to the
    database, therefore, subsequent plotting or printing of solutions
    is not possible.  A SET command must be issued after the RSTMAC
    command to post-process each solution.

    RSTMAC comparison on cyclic symmetry analysis works only if the
    number of sectors on File1 and File2 are the same. Also comparison
    cannot be made between cyclic symmetry results and full 360 degree
    model results (File1 – cyclic solution, File2 – full 360 degree
    model solution).  Comparing cyclic symmetry solutions written on
    selected set of node (OUTRES) is not supported.

    The modal assurance criterion values can be retrieved as
    parameters using the *GET command (Entity = RSTMAC).

    For more information and an example, see Comparing Nodal Solutions
    From Two Models (RSTMAC) in the Basic Analysis Guide.
    """
    command = f"RSTMAC,{file1},{lstep1},{sbstep1},{file2},{lstep2},{sbstep2},,{maclim},{cname},{keyprint}"
    return self.run(command, **kwargs)



def spoint(self, node="", x="", y="", z="", **kwargs):
    """Defines a point for moment summations.

    APDL Command: SPOINT

    Parameters
    ----------
    node
        Node number of the desired point.  If zero, use X,Y,Z to describe
        point.

    x, y, z
        Global Cartesian coordinates of the desired summation point.  Used
        if NODE is 0.  Defaults to (0,0,0).

    Notes
    -----
    Defines a point (any point other than the origin) about which the
    tabular moment summations are computed [NFORCE, FSUM].  If force
    summations are desired in other than the global Cartesian directions, a
    node number must be specified on the NODE field, and the desired
    coordinate system must be activated with RSYS.
    """
    command = f"SPOINT,{node},{x},{y},{z}"
    return self.run(command, **kwargs)


def spmwrite(self, method="", nmode="", inputs="", inputlabels="",
             outputs="", outputlabels="", nic="", velacckey="",
             fileformat="", **kwargs):
    """Calculates the state-space matrices and writes them to the SPM file.

    APDL Command: SPMWRITE

    Parameters
    ----------
    method
        Reduction method for the calculation of the state-space matrices.

        MODAL - Method based on modal analysis results from LANB, LANPCG, SNODE, or SUBSP
                eigensolver (default).

    nmode
        Number of modes to be used. Defaults to all modes.

    inputs
        Definition of the inputs. Defaults to all load vectors on the MODE
        file.

    inputlabels
        Definition of the input labels. Defaults to the load vector numbers
        or input definition (node and degree of freedom array parameter),
        depending on the Inputs specification.

    outputs
        Definition of the outputs. Defaults to the inputs.

    outputlabels
        Definition of the output labels. Defaults to the output definition
        (node and degree of freedom) if used, else defaults to the
        InputLabels.

    nic
        Load vector on the MODE file used for the calculation of the
        initial conditions. Defaults to no initial condition.

    velacckey
        Output velocities and accelerations key.

        OFF - Output displacements only (default).

        ON - Output displacements, velocities and accelerations.

    fileformat
        The format of the SPM file.

        0 - Dense format.

        1 - Matrix Market Exchange format (non-zero terms only).

        2 - Simplorer SML format without reference (default).

        3 - Simplorer SML format with common reference.

        4 - Simplorer SML format with independent references.

    Notes
    -----
    The SPMWRITE generates the file Jobname.SPM containing the state-space
    matrices and other information.

    The following applies to the SML formats (FileFormat = 2, 3, and 4):

    For conservative systems where the outputs are equal to the inputs
    (Outputs is left blank):

    The labels for the inputs (InputLabels) are required.

    The Inputs must use the array parameter option so that the input
    degrees of freedom (DOFs) are known.

    For non-conservative systems where the outputs are not equal to the
    inputs:

    The labels for the outputs (OutputLabels) are required.

    The file formats with references (FileFormat = 3 and 4) do not apply.

    Velocity and acceleration results are not included in the state-space
    matrices calculation (VelAccKey = OFF)

    File format with common reference (FileFormat = 3) does not apply if
    the inputs are based on DOFs of a different nature.  All input DOFs
    must be either all rotational or all translational and not a mix of the
    two.

    A graphics file (Jobname_SPM.PNG) is generated. It contains an element
    plot of the model.

    For more details about the reduction method and the generation of the
    state-space matrices, see Reduced-Order Modeling for State-Space
    Matrices Export in the Mechanical APDL Theory Reference.

    For examples of the command usage, see State-Space Matrices Export.
    """
    command = f"SPMWRITE,{method},{nmode},{inputs},{inputlabels},{outputs},{outputlabels},{nic},{velacckey},{fileformat}"
    return self.run(command, **kwargs)
